/*************************************************************************
 *                                                                       *
 *                                UTILS.C                                *
 *                                                                       *
 *************************************************************************

                           Copyright 1996 Echidna

   DESCRIPTION
		Stuff that's generally useful but doesn't deserve it's own
		library

   PROGRAMMERS
		Gregg A. Tavares

   FUNCTIONS

   TABS : 5 9

   HISTORY
		07/11/96 GAT: Created based on echidna stuff by R.G. Marquez

 *************************************************************************/

/**************************** I N C L U D E S ****************************/

#include "platform.h"
#include "switches.h"
#include "echidna\ensure.h"
#include "echidna\utils.h"

/*************************** C O N S T A N T S ***************************/

#define INIT_STATIC_SEED	(100001L)
#define STATIC_DIV			(2796203L)

/******************************* T Y P E S *******************************/


/************************** P R O T O T Y P E S **************************/

int32 UTL_COS (uint8 angle);
int32 UTL_SIN (uint8 angle);

/***************************** G L O B A L S *****************************/

#define	UTL_SINVALUE(v)	((float)((v) / 65536.0))
#define UTL_SINFLOAT(v)	((float)(v))


// 0 - 90 degrees
int32 gSinTable64[] =
{
	0x00000000,
	0x00000648,
	0x00000C8F,
	0x000012D5,
	0x00001917,
	0x00001F56,
	0x00002590,
	0x00002BC4,
	0x000031F1,
	0x00003817,
	0x00003E33,
	0x00004447,
	0x00004A50,
	0x0000504D,
	0x0000563E,
	0x00005C22,
	0x000061F7,
	0x000067BD,
	0x00006D74,
	0x00007319,
	0x000078AD,
	0x00007E2E,
	0x0000839C,
	0x000088F5,
	0x00008E39,
	0x00009368,
	0x0000987F,
	0x00009D7F,
	0x0000A267,
	0x0000A736,
	0x0000ABEB,
	0x0000B085,
	0x0000B504,
	0x0000B968,
	0x0000BDAE,
	0x0000C1D8,
	0x0000C5E4,
	0x0000C9D1,
	0x0000CD9F,
	0x0000D14D,
	0x0000D4DB,
	0x0000D848,
	0x0000DB94,
	0x0000DEBE,
	0x0000E1C5,
	0x0000E4AA,
	0x0000E76B,
	0x0000EA09,
	0x0000EC83,
	0x0000EED8,
	0x0000F109,
	0x0000F314,
	0x0000F4FA,
	0x0000F6BA,
	0x0000F853,
	0x0000F9C7,
	0x0000FB14,
	0x0000FC3B,
	0x0000FD3A,
	0x0000FE13,
	0x0000FEC4,
	0x0000FF4E,
	0x0000FFB1,
	0x0000FFEC,
	0x00010000,
};

#if 0
float gSinTable256[] =
{
	UTL_SINFLOAT(0.000000),
	UTL_SINFLOAT(0.006136),
	UTL_SINFLOAT(0.012272),
	UTL_SINFLOAT(0.018407),
	UTL_SINFLOAT(0.024541),
	UTL_SINFLOAT(0.030675),
	UTL_SINFLOAT(0.036807),
	UTL_SINFLOAT(0.042938),
	UTL_SINFLOAT(0.049068),
	UTL_SINFLOAT(0.055195),
	UTL_SINFLOAT(0.061321),
	UTL_SINFLOAT(0.067444),
	UTL_SINFLOAT(0.073565),
	UTL_SINFLOAT(0.079682),
	UTL_SINFLOAT(0.085797),
	UTL_SINFLOAT(0.091909),
	UTL_SINFLOAT(0.098017),
	UTL_SINFLOAT(0.104122),
	UTL_SINFLOAT(0.110222),
	UTL_SINFLOAT(0.116319),
	UTL_SINFLOAT(0.122411),
	UTL_SINFLOAT(0.128498),
	UTL_SINFLOAT(0.134581),
	UTL_SINFLOAT(0.140658),
	UTL_SINFLOAT(0.146730),
	UTL_SINFLOAT(0.152797),
	UTL_SINFLOAT(0.158858),
	UTL_SINFLOAT(0.164913),
	UTL_SINFLOAT(0.170962),
	UTL_SINFLOAT(0.177004),
	UTL_SINFLOAT(0.183040),
	UTL_SINFLOAT(0.189069),
	UTL_SINFLOAT(0.195090),
	UTL_SINFLOAT(0.201105),
	UTL_SINFLOAT(0.207111),
	UTL_SINFLOAT(0.213110),
	UTL_SINFLOAT(0.219101),
	UTL_SINFLOAT(0.225084),
	UTL_SINFLOAT(0.231058),
	UTL_SINFLOAT(0.237024),
	UTL_SINFLOAT(0.242980),
	UTL_SINFLOAT(0.248928),
	UTL_SINFLOAT(0.254866),
	UTL_SINFLOAT(0.260794),
	UTL_SINFLOAT(0.266713),
	UTL_SINFLOAT(0.272621),
	UTL_SINFLOAT(0.278520),
	UTL_SINFLOAT(0.284408),
	UTL_SINFLOAT(0.290285),
	UTL_SINFLOAT(0.296151),
	UTL_SINFLOAT(0.302006),
	UTL_SINFLOAT(0.307850),
	UTL_SINFLOAT(0.313682),
	UTL_SINFLOAT(0.319502),
	UTL_SINFLOAT(0.325310),
	UTL_SINFLOAT(0.331106),
	UTL_SINFLOAT(0.336890),
	UTL_SINFLOAT(0.342661),
	UTL_SINFLOAT(0.348419),
	UTL_SINFLOAT(0.354164),
	UTL_SINFLOAT(0.359895),
	UTL_SINFLOAT(0.365613),
	UTL_SINFLOAT(0.371317),
	UTL_SINFLOAT(0.377007),
	UTL_SINFLOAT(0.382683),
	UTL_SINFLOAT(0.388345),
	UTL_SINFLOAT(0.393992),
	UTL_SINFLOAT(0.399624),
	UTL_SINFLOAT(0.405241),
	UTL_SINFLOAT(0.410843),
	UTL_SINFLOAT(0.416430),
	UTL_SINFLOAT(0.422000),
	UTL_SINFLOAT(0.427555),
	UTL_SINFLOAT(0.433094),
	UTL_SINFLOAT(0.438616),
	UTL_SINFLOAT(0.444122),
	UTL_SINFLOAT(0.449611),
	UTL_SINFLOAT(0.455084),
	UTL_SINFLOAT(0.460539),
	UTL_SINFLOAT(0.465976),
	UTL_SINFLOAT(0.471397),
	UTL_SINFLOAT(0.476799),
	UTL_SINFLOAT(0.482184),
	UTL_SINFLOAT(0.487550),
	UTL_SINFLOAT(0.492898),
	UTL_SINFLOAT(0.498228),
	UTL_SINFLOAT(0.503538),
	UTL_SINFLOAT(0.508830),
	UTL_SINFLOAT(0.514103),
	UTL_SINFLOAT(0.519356),
	UTL_SINFLOAT(0.524590),
	UTL_SINFLOAT(0.529804),
	UTL_SINFLOAT(0.534998),
	UTL_SINFLOAT(0.540171),
	UTL_SINFLOAT(0.545325),
	UTL_SINFLOAT(0.550458),
	UTL_SINFLOAT(0.555570),
	UTL_SINFLOAT(0.560662),
	UTL_SINFLOAT(0.565732),
	UTL_SINFLOAT(0.570781),
	UTL_SINFLOAT(0.575808),
	UTL_SINFLOAT(0.580814),
	UTL_SINFLOAT(0.585798),
	UTL_SINFLOAT(0.590760),
	UTL_SINFLOAT(0.595699),
	UTL_SINFLOAT(0.600616),
	UTL_SINFLOAT(0.605511),
	UTL_SINFLOAT(0.610383),
	UTL_SINFLOAT(0.615232),
	UTL_SINFLOAT(0.620057),
	UTL_SINFLOAT(0.624859),
	UTL_SINFLOAT(0.629638),
	UTL_SINFLOAT(0.634393),
	UTL_SINFLOAT(0.639124),
	UTL_SINFLOAT(0.643832),
	UTL_SINFLOAT(0.648514),
	UTL_SINFLOAT(0.653173),
	UTL_SINFLOAT(0.657807),
	UTL_SINFLOAT(0.662416),
	UTL_SINFLOAT(0.667000),
	UTL_SINFLOAT(0.671559),
	UTL_SINFLOAT(0.676093),
	UTL_SINFLOAT(0.680601),
	UTL_SINFLOAT(0.685084),
	UTL_SINFLOAT(0.689541),
	UTL_SINFLOAT(0.693971),
	UTL_SINFLOAT(0.698376),
	UTL_SINFLOAT(0.702755),
	UTL_SINFLOAT(0.707107),
	UTL_SINFLOAT(0.711432),
	UTL_SINFLOAT(0.715731),
	UTL_SINFLOAT(0.720003),
	UTL_SINFLOAT(0.724247),
	UTL_SINFLOAT(0.728464),
	UTL_SINFLOAT(0.732654),
	UTL_SINFLOAT(0.736817),
	UTL_SINFLOAT(0.740951),
	UTL_SINFLOAT(0.745058),
	UTL_SINFLOAT(0.749136),
	UTL_SINFLOAT(0.753187),
	UTL_SINFLOAT(0.757209),
	UTL_SINFLOAT(0.761202),
	UTL_SINFLOAT(0.765167),
	UTL_SINFLOAT(0.769103),
	UTL_SINFLOAT(0.773010),
	UTL_SINFLOAT(0.776888),
	UTL_SINFLOAT(0.780737),
	UTL_SINFLOAT(0.784557),
	UTL_SINFLOAT(0.788346),
	UTL_SINFLOAT(0.792107),
	UTL_SINFLOAT(0.795837),
	UTL_SINFLOAT(0.799537),
	UTL_SINFLOAT(0.803208),
	UTL_SINFLOAT(0.806848),
	UTL_SINFLOAT(0.810457),
	UTL_SINFLOAT(0.814036),
	UTL_SINFLOAT(0.817585),
	UTL_SINFLOAT(0.821103),
	UTL_SINFLOAT(0.824589),
	UTL_SINFLOAT(0.828045),
	UTL_SINFLOAT(0.831470),
	UTL_SINFLOAT(0.834863),
	UTL_SINFLOAT(0.838225),
	UTL_SINFLOAT(0.841555),
	UTL_SINFLOAT(0.844854),
	UTL_SINFLOAT(0.848120),
	UTL_SINFLOAT(0.851355),
	UTL_SINFLOAT(0.854558),
	UTL_SINFLOAT(0.857729),
	UTL_SINFLOAT(0.860867),
	UTL_SINFLOAT(0.863973),
	UTL_SINFLOAT(0.867046),
	UTL_SINFLOAT(0.870087),
	UTL_SINFLOAT(0.873095),
	UTL_SINFLOAT(0.876070),
	UTL_SINFLOAT(0.879012),
	UTL_SINFLOAT(0.881921),
	UTL_SINFLOAT(0.884797),
	UTL_SINFLOAT(0.887640),
	UTL_SINFLOAT(0.890449),
	UTL_SINFLOAT(0.893224),
	UTL_SINFLOAT(0.895966),
	UTL_SINFLOAT(0.898674),
	UTL_SINFLOAT(0.901349),
	UTL_SINFLOAT(0.903989),
	UTL_SINFLOAT(0.906596),
	UTL_SINFLOAT(0.909168),
	UTL_SINFLOAT(0.911706),
	UTL_SINFLOAT(0.914210),
	UTL_SINFLOAT(0.916679),
	UTL_SINFLOAT(0.919114),
	UTL_SINFLOAT(0.921514),
	UTL_SINFLOAT(0.923880),
	UTL_SINFLOAT(0.926210),
	UTL_SINFLOAT(0.928506),
	UTL_SINFLOAT(0.930767),
	UTL_SINFLOAT(0.932993),
	UTL_SINFLOAT(0.935184),
	UTL_SINFLOAT(0.937339),
	UTL_SINFLOAT(0.939459),
	UTL_SINFLOAT(0.941544),
	UTL_SINFLOAT(0.943593),
	UTL_SINFLOAT(0.945607),
	UTL_SINFLOAT(0.947586),
	UTL_SINFLOAT(0.949528),
	UTL_SINFLOAT(0.951435),
	UTL_SINFLOAT(0.953306),
	UTL_SINFLOAT(0.955141),
	UTL_SINFLOAT(0.956940),
	UTL_SINFLOAT(0.958703),
	UTL_SINFLOAT(0.960431),
	UTL_SINFLOAT(0.962121),
	UTL_SINFLOAT(0.963776),
	UTL_SINFLOAT(0.965394),
	UTL_SINFLOAT(0.966976),
	UTL_SINFLOAT(0.968522),
	UTL_SINFLOAT(0.970031),
	UTL_SINFLOAT(0.971504),
	UTL_SINFLOAT(0.972940),
	UTL_SINFLOAT(0.974339),
	UTL_SINFLOAT(0.975702),
	UTL_SINFLOAT(0.977028),
	UTL_SINFLOAT(0.978317),
	UTL_SINFLOAT(0.979570),
	UTL_SINFLOAT(0.980785),
	UTL_SINFLOAT(0.981964),
	UTL_SINFLOAT(0.983105),
	UTL_SINFLOAT(0.984210),
	UTL_SINFLOAT(0.985278),
	UTL_SINFLOAT(0.986308),
	UTL_SINFLOAT(0.987301),
	UTL_SINFLOAT(0.988258),
	UTL_SINFLOAT(0.989177),
	UTL_SINFLOAT(0.990058),
	UTL_SINFLOAT(0.990903),
	UTL_SINFLOAT(0.991710),
	UTL_SINFLOAT(0.992480),
	UTL_SINFLOAT(0.993212),
	UTL_SINFLOAT(0.993907),
	UTL_SINFLOAT(0.994565),
	UTL_SINFLOAT(0.995185),
	UTL_SINFLOAT(0.995767),
	UTL_SINFLOAT(0.996313),
	UTL_SINFLOAT(0.996820),
	UTL_SINFLOAT(0.997290),
	UTL_SINFLOAT(0.997723),
	UTL_SINFLOAT(0.998118),
	UTL_SINFLOAT(0.998476),
	UTL_SINFLOAT(0.998795),
	UTL_SINFLOAT(0.999078),
	UTL_SINFLOAT(0.999322),
	UTL_SINFLOAT(0.999529),
	UTL_SINFLOAT(0.999699),
	UTL_SINFLOAT(0.999831),
	UTL_SINFLOAT(0.999925),
	UTL_SINFLOAT(0.999981),
	UTL_SINFLOAT(1.000000),
};
#endif

float gSinTable1024[] =
{
	UTL_SINFLOAT(0.000000),		// 0
	UTL_SINFLOAT(0.006136),		// 1
	UTL_SINFLOAT(0.012272),		// 2
	UTL_SINFLOAT(0.018407),		// 3
	UTL_SINFLOAT(0.024541),		// 4
	UTL_SINFLOAT(0.030675),		// 5
	UTL_SINFLOAT(0.036807),		// 6
	UTL_SINFLOAT(0.042938),		// 7
	UTL_SINFLOAT(0.049068),		// 8
	UTL_SINFLOAT(0.055195),		// 9
	UTL_SINFLOAT(0.061321),		// 10
	UTL_SINFLOAT(0.067444),		// 11
	UTL_SINFLOAT(0.073565),		// 12
	UTL_SINFLOAT(0.079682),		// 13
	UTL_SINFLOAT(0.085797),		// 14
	UTL_SINFLOAT(0.091909),		// 15
	UTL_SINFLOAT(0.098017),		// 16
	UTL_SINFLOAT(0.104122),		// 17
	UTL_SINFLOAT(0.110222),		// 18
	UTL_SINFLOAT(0.116319),		// 19
	UTL_SINFLOAT(0.122411),		// 20
	UTL_SINFLOAT(0.128498),		// 21
	UTL_SINFLOAT(0.134581),		// 22
	UTL_SINFLOAT(0.140658),		// 23
	UTL_SINFLOAT(0.146730),		// 24
	UTL_SINFLOAT(0.152797),		// 25
	UTL_SINFLOAT(0.158858),		// 26
	UTL_SINFLOAT(0.164913),		// 27
	UTL_SINFLOAT(0.170962),		// 28
	UTL_SINFLOAT(0.177004),		// 29
	UTL_SINFLOAT(0.183040),		// 30
	UTL_SINFLOAT(0.189069),		// 31
	UTL_SINFLOAT(0.195090),		// 32
	UTL_SINFLOAT(0.201105),		// 33
	UTL_SINFLOAT(0.207111),		// 34
	UTL_SINFLOAT(0.213110),		// 35
	UTL_SINFLOAT(0.219101),		// 36
	UTL_SINFLOAT(0.225084),		// 37
	UTL_SINFLOAT(0.231058),		// 38
	UTL_SINFLOAT(0.237024),		// 39
	UTL_SINFLOAT(0.242980),		// 40
	UTL_SINFLOAT(0.248928),		// 41
	UTL_SINFLOAT(0.254866),		// 42
	UTL_SINFLOAT(0.260794),		// 43
	UTL_SINFLOAT(0.266713),		// 44
	UTL_SINFLOAT(0.272621),		// 45
	UTL_SINFLOAT(0.278520),		// 46
	UTL_SINFLOAT(0.284408),		// 47
	UTL_SINFLOAT(0.290285),		// 48
	UTL_SINFLOAT(0.296151),		// 49
	UTL_SINFLOAT(0.302006),		// 50
	UTL_SINFLOAT(0.307850),		// 51
	UTL_SINFLOAT(0.313682),		// 52
	UTL_SINFLOAT(0.319502),		// 53
	UTL_SINFLOAT(0.325310),		// 54
	UTL_SINFLOAT(0.331106),		// 55
	UTL_SINFLOAT(0.336890),		// 56
	UTL_SINFLOAT(0.342661),		// 57
	UTL_SINFLOAT(0.348419),		// 58
	UTL_SINFLOAT(0.354164),		// 59
	UTL_SINFLOAT(0.359895),		// 60
	UTL_SINFLOAT(0.365613),		// 61
	UTL_SINFLOAT(0.371317),		// 62
	UTL_SINFLOAT(0.377007),		// 63
	UTL_SINFLOAT(0.382683),		// 64
	UTL_SINFLOAT(0.388345),		// 65
	UTL_SINFLOAT(0.393992),		// 66
	UTL_SINFLOAT(0.399624),		// 67
	UTL_SINFLOAT(0.405241),		// 68
	UTL_SINFLOAT(0.410843),		// 69
	UTL_SINFLOAT(0.416430),		// 70
	UTL_SINFLOAT(0.422000),		// 71
	UTL_SINFLOAT(0.427555),		// 72
	UTL_SINFLOAT(0.433094),		// 73
	UTL_SINFLOAT(0.438616),		// 74
	UTL_SINFLOAT(0.444122),		// 75
	UTL_SINFLOAT(0.449611),		// 76
	UTL_SINFLOAT(0.455084),		// 77
	UTL_SINFLOAT(0.460539),		// 78
	UTL_SINFLOAT(0.465976),		// 79
	UTL_SINFLOAT(0.471397),		// 80
	UTL_SINFLOAT(0.476799),		// 81
	UTL_SINFLOAT(0.482184),		// 82
	UTL_SINFLOAT(0.487550),		// 83
	UTL_SINFLOAT(0.492898),		// 84
	UTL_SINFLOAT(0.498228),		// 85
	UTL_SINFLOAT(0.503538),		// 86
	UTL_SINFLOAT(0.508830),		// 87
	UTL_SINFLOAT(0.514103),		// 88
	UTL_SINFLOAT(0.519356),		// 89
	UTL_SINFLOAT(0.524590),		// 90
	UTL_SINFLOAT(0.529804),		// 91
	UTL_SINFLOAT(0.534998),		// 92
	UTL_SINFLOAT(0.540171),		// 93
	UTL_SINFLOAT(0.545325),		// 94
	UTL_SINFLOAT(0.550458),		// 95
	UTL_SINFLOAT(0.555570),		// 96
	UTL_SINFLOAT(0.560662),		// 97
	UTL_SINFLOAT(0.565732),		// 98
	UTL_SINFLOAT(0.570781),		// 99
	UTL_SINFLOAT(0.575808),		// 100
	UTL_SINFLOAT(0.580814),		// 101
	UTL_SINFLOAT(0.585798),		// 102
	UTL_SINFLOAT(0.590760),		// 103
	UTL_SINFLOAT(0.595699),		// 104
	UTL_SINFLOAT(0.600616),		// 105
	UTL_SINFLOAT(0.605511),		// 106
	UTL_SINFLOAT(0.610383),		// 107
	UTL_SINFLOAT(0.615232),		// 108
	UTL_SINFLOAT(0.620057),		// 109
	UTL_SINFLOAT(0.624859),		// 110
	UTL_SINFLOAT(0.629638),		// 111
	UTL_SINFLOAT(0.634393),		// 112
	UTL_SINFLOAT(0.639124),		// 113
	UTL_SINFLOAT(0.643832),		// 114
	UTL_SINFLOAT(0.648514),		// 115
	UTL_SINFLOAT(0.653173),		// 116
	UTL_SINFLOAT(0.657807),		// 117
	UTL_SINFLOAT(0.662416),		// 118
	UTL_SINFLOAT(0.667000),		// 119
	UTL_SINFLOAT(0.671559),		// 120
	UTL_SINFLOAT(0.676093),		// 121
	UTL_SINFLOAT(0.680601),		// 122
	UTL_SINFLOAT(0.685084),		// 123
	UTL_SINFLOAT(0.689541),		// 124
	UTL_SINFLOAT(0.693971),		// 125
	UTL_SINFLOAT(0.698376),		// 126
	UTL_SINFLOAT(0.702755),		// 127
	UTL_SINFLOAT(0.707107),		// 128
	UTL_SINFLOAT(0.711432),		// 129
	UTL_SINFLOAT(0.715731),		// 130
	UTL_SINFLOAT(0.720003),		// 131
	UTL_SINFLOAT(0.724247),		// 132
	UTL_SINFLOAT(0.728464),		// 133
	UTL_SINFLOAT(0.732654),		// 134
	UTL_SINFLOAT(0.736817),		// 135
	UTL_SINFLOAT(0.740951),		// 136
	UTL_SINFLOAT(0.745058),		// 137
	UTL_SINFLOAT(0.749136),		// 138
	UTL_SINFLOAT(0.753187),		// 139
	UTL_SINFLOAT(0.757209),		// 140
	UTL_SINFLOAT(0.761202),		// 141
	UTL_SINFLOAT(0.765167),		// 142
	UTL_SINFLOAT(0.769103),		// 143
	UTL_SINFLOAT(0.773010),		// 144
	UTL_SINFLOAT(0.776888),		// 145
	UTL_SINFLOAT(0.780737),		// 146
	UTL_SINFLOAT(0.784557),		// 147
	UTL_SINFLOAT(0.788346),		// 148
	UTL_SINFLOAT(0.792107),		// 149
	UTL_SINFLOAT(0.795837),		// 150
	UTL_SINFLOAT(0.799537),		// 151
	UTL_SINFLOAT(0.803208),		// 152
	UTL_SINFLOAT(0.806848),		// 153
	UTL_SINFLOAT(0.810457),		// 154
	UTL_SINFLOAT(0.814036),		// 155
	UTL_SINFLOAT(0.817585),		// 156
	UTL_SINFLOAT(0.821103),		// 157
	UTL_SINFLOAT(0.824589),		// 158
	UTL_SINFLOAT(0.828045),		// 159
	UTL_SINFLOAT(0.831470),		// 160
	UTL_SINFLOAT(0.834863),		// 161
	UTL_SINFLOAT(0.838225),		// 162
	UTL_SINFLOAT(0.841555),		// 163
	UTL_SINFLOAT(0.844854),		// 164
	UTL_SINFLOAT(0.848120),		// 165
	UTL_SINFLOAT(0.851355),		// 166
	UTL_SINFLOAT(0.854558),		// 167
	UTL_SINFLOAT(0.857729),		// 168
	UTL_SINFLOAT(0.860867),		// 169
	UTL_SINFLOAT(0.863973),		// 170
	UTL_SINFLOAT(0.867046),		// 171
	UTL_SINFLOAT(0.870087),		// 172
	UTL_SINFLOAT(0.873095),		// 173
	UTL_SINFLOAT(0.876070),		// 174
	UTL_SINFLOAT(0.879012),		// 175
	UTL_SINFLOAT(0.881921),		// 176
	UTL_SINFLOAT(0.884797),		// 177
	UTL_SINFLOAT(0.887640),		// 178
	UTL_SINFLOAT(0.890449),		// 179
	UTL_SINFLOAT(0.893224),		// 180
	UTL_SINFLOAT(0.895966),		// 181
	UTL_SINFLOAT(0.898674),		// 182
	UTL_SINFLOAT(0.901349),		// 183
	UTL_SINFLOAT(0.903989),		// 184
	UTL_SINFLOAT(0.906596),		// 185
	UTL_SINFLOAT(0.909168),		// 186
	UTL_SINFLOAT(0.911706),		// 187
	UTL_SINFLOAT(0.914210),		// 188
	UTL_SINFLOAT(0.916679),		// 189
	UTL_SINFLOAT(0.919114),		// 190
	UTL_SINFLOAT(0.921514),		// 191
	UTL_SINFLOAT(0.923880),		// 192
	UTL_SINFLOAT(0.926210),		// 193
	UTL_SINFLOAT(0.928506),		// 194
	UTL_SINFLOAT(0.930767),		// 195
	UTL_SINFLOAT(0.932993),		// 196
	UTL_SINFLOAT(0.935184),		// 197
	UTL_SINFLOAT(0.937339),		// 198
	UTL_SINFLOAT(0.939459),		// 199
	UTL_SINFLOAT(0.941544),		// 200
	UTL_SINFLOAT(0.943593),		// 201
	UTL_SINFLOAT(0.945607),		// 202
	UTL_SINFLOAT(0.947586),		// 203
	UTL_SINFLOAT(0.949528),		// 204
	UTL_SINFLOAT(0.951435),		// 205
	UTL_SINFLOAT(0.953306),		// 206
	UTL_SINFLOAT(0.955141),		// 207
	UTL_SINFLOAT(0.956940),		// 208
	UTL_SINFLOAT(0.958703),		// 209
	UTL_SINFLOAT(0.960431),		// 210
	UTL_SINFLOAT(0.962121),		// 211
	UTL_SINFLOAT(0.963776),		// 212
	UTL_SINFLOAT(0.965394),		// 213
	UTL_SINFLOAT(0.966976),		// 214
	UTL_SINFLOAT(0.968522),		// 215
	UTL_SINFLOAT(0.970031),		// 216
	UTL_SINFLOAT(0.971504),		// 217
	UTL_SINFLOAT(0.972940),		// 218
	UTL_SINFLOAT(0.974339),		// 219
	UTL_SINFLOAT(0.975702),		// 220
	UTL_SINFLOAT(0.977028),		// 221
	UTL_SINFLOAT(0.978317),		// 222
	UTL_SINFLOAT(0.979570),		// 223
	UTL_SINFLOAT(0.980785),		// 224
	UTL_SINFLOAT(0.981964),		// 225
	UTL_SINFLOAT(0.983105),		// 226
	UTL_SINFLOAT(0.984210),		// 227
	UTL_SINFLOAT(0.985278),		// 228
	UTL_SINFLOAT(0.986308),		// 229
	UTL_SINFLOAT(0.987301),		// 230
	UTL_SINFLOAT(0.988258),		// 231
	UTL_SINFLOAT(0.989177),		// 232
	UTL_SINFLOAT(0.990058),		// 233
	UTL_SINFLOAT(0.990903),		// 234
	UTL_SINFLOAT(0.991710),		// 235
	UTL_SINFLOAT(0.992480),		// 236
	UTL_SINFLOAT(0.993212),		// 237
	UTL_SINFLOAT(0.993907),		// 238
	UTL_SINFLOAT(0.994565),		// 239
	UTL_SINFLOAT(0.995185),		// 240
	UTL_SINFLOAT(0.995767),		// 241
	UTL_SINFLOAT(0.996313),		// 242
	UTL_SINFLOAT(0.996820),		// 243
	UTL_SINFLOAT(0.997290),		// 244
	UTL_SINFLOAT(0.997723),		// 245
	UTL_SINFLOAT(0.998118),		// 246
	UTL_SINFLOAT(0.998476),		// 247
	UTL_SINFLOAT(0.998795),		// 248
	UTL_SINFLOAT(0.999078),		// 249
	UTL_SINFLOAT(0.999322),		// 250
	UTL_SINFLOAT(0.999529),		// 251
	UTL_SINFLOAT(0.999699),		// 252
	UTL_SINFLOAT(0.999831),		// 253
	UTL_SINFLOAT(0.999925),		// 254
	UTL_SINFLOAT(0.999981),		// 255
	UTL_SINFLOAT(1.000000),		// 256
	UTL_SINFLOAT(0.999981),		// 257
	UTL_SINFLOAT(0.999925),		// 258
	UTL_SINFLOAT(0.999831),		// 259
	UTL_SINFLOAT(0.999699),		// 260
	UTL_SINFLOAT(0.999529),		// 261
	UTL_SINFLOAT(0.999322),		// 262
	UTL_SINFLOAT(0.999078),		// 263
	UTL_SINFLOAT(0.998795),		// 264
	UTL_SINFLOAT(0.998476),		// 265
	UTL_SINFLOAT(0.998118),		// 266
	UTL_SINFLOAT(0.997723),		// 267
	UTL_SINFLOAT(0.997290),		// 268
	UTL_SINFLOAT(0.996820),		// 269
	UTL_SINFLOAT(0.996313),		// 270
	UTL_SINFLOAT(0.995767),		// 271
	UTL_SINFLOAT(0.995185),		// 272
	UTL_SINFLOAT(0.994565),		// 273
	UTL_SINFLOAT(0.993907),		// 274
	UTL_SINFLOAT(0.993212),		// 275
	UTL_SINFLOAT(0.992480),		// 276
	UTL_SINFLOAT(0.991710),		// 277
	UTL_SINFLOAT(0.990903),		// 278
	UTL_SINFLOAT(0.990058),		// 279
	UTL_SINFLOAT(0.989177),		// 280
	UTL_SINFLOAT(0.988258),		// 281
	UTL_SINFLOAT(0.987301),		// 282
	UTL_SINFLOAT(0.986308),		// 283
	UTL_SINFLOAT(0.985278),		// 284
	UTL_SINFLOAT(0.984210),		// 285
	UTL_SINFLOAT(0.983105),		// 286
	UTL_SINFLOAT(0.981964),		// 287
	UTL_SINFLOAT(0.980785),		// 288
	UTL_SINFLOAT(0.979570),		// 289
	UTL_SINFLOAT(0.978317),		// 290
	UTL_SINFLOAT(0.977028),		// 291
	UTL_SINFLOAT(0.975702),		// 292
	UTL_SINFLOAT(0.974339),		// 293
	UTL_SINFLOAT(0.972940),		// 294
	UTL_SINFLOAT(0.971504),		// 295
	UTL_SINFLOAT(0.970031),		// 296
	UTL_SINFLOAT(0.968522),		// 297
	UTL_SINFLOAT(0.966976),		// 298
	UTL_SINFLOAT(0.965394),		// 299
	UTL_SINFLOAT(0.963776),		// 300
	UTL_SINFLOAT(0.962121),		// 301
	UTL_SINFLOAT(0.960431),		// 302
	UTL_SINFLOAT(0.958703),		// 303
	UTL_SINFLOAT(0.956940),		// 304
	UTL_SINFLOAT(0.955141),		// 305
	UTL_SINFLOAT(0.953306),		// 306
	UTL_SINFLOAT(0.951435),		// 307
	UTL_SINFLOAT(0.949528),		// 308
	UTL_SINFLOAT(0.947586),		// 309
	UTL_SINFLOAT(0.945607),		// 310
	UTL_SINFLOAT(0.943593),		// 311
	UTL_SINFLOAT(0.941544),		// 312
	UTL_SINFLOAT(0.939459),		// 313
	UTL_SINFLOAT(0.937339),		// 314
	UTL_SINFLOAT(0.935184),		// 315
	UTL_SINFLOAT(0.932993),		// 316
	UTL_SINFLOAT(0.930767),		// 317
	UTL_SINFLOAT(0.928506),		// 318
	UTL_SINFLOAT(0.926210),		// 319
	UTL_SINFLOAT(0.923880),		// 320
	UTL_SINFLOAT(0.921514),		// 321
	UTL_SINFLOAT(0.919114),		// 322
	UTL_SINFLOAT(0.916679),		// 323
	UTL_SINFLOAT(0.914210),		// 324
	UTL_SINFLOAT(0.911706),		// 325
	UTL_SINFLOAT(0.909168),		// 326
	UTL_SINFLOAT(0.906596),		// 327
	UTL_SINFLOAT(0.903989),		// 328
	UTL_SINFLOAT(0.901349),		// 329
	UTL_SINFLOAT(0.898674),		// 330
	UTL_SINFLOAT(0.895966),		// 331
	UTL_SINFLOAT(0.893224),		// 332
	UTL_SINFLOAT(0.890449),		// 333
	UTL_SINFLOAT(0.887640),		// 334
	UTL_SINFLOAT(0.884797),		// 335
	UTL_SINFLOAT(0.881921),		// 336
	UTL_SINFLOAT(0.879012),		// 337
	UTL_SINFLOAT(0.876070),		// 338
	UTL_SINFLOAT(0.873095),		// 339
	UTL_SINFLOAT(0.870087),		// 340
	UTL_SINFLOAT(0.867046),		// 341
	UTL_SINFLOAT(0.863973),		// 342
	UTL_SINFLOAT(0.860867),		// 343
	UTL_SINFLOAT(0.857729),		// 344
	UTL_SINFLOAT(0.854558),		// 345
	UTL_SINFLOAT(0.851355),		// 346
	UTL_SINFLOAT(0.848120),		// 347
	UTL_SINFLOAT(0.844854),		// 348
	UTL_SINFLOAT(0.841555),		// 349
	UTL_SINFLOAT(0.838225),		// 350
	UTL_SINFLOAT(0.834863),		// 351
	UTL_SINFLOAT(0.831470),		// 352
	UTL_SINFLOAT(0.828045),		// 353
	UTL_SINFLOAT(0.824589),		// 354
	UTL_SINFLOAT(0.821103),		// 355
	UTL_SINFLOAT(0.817585),		// 356
	UTL_SINFLOAT(0.814036),		// 357
	UTL_SINFLOAT(0.810457),		// 358
	UTL_SINFLOAT(0.806848),		// 359
	UTL_SINFLOAT(0.803208),		// 360
	UTL_SINFLOAT(0.799537),		// 361
	UTL_SINFLOAT(0.795837),		// 362
	UTL_SINFLOAT(0.792107),		// 363
	UTL_SINFLOAT(0.788346),		// 364
	UTL_SINFLOAT(0.784557),		// 365
	UTL_SINFLOAT(0.780737),		// 366
	UTL_SINFLOAT(0.776888),		// 367
	UTL_SINFLOAT(0.773010),		// 368
	UTL_SINFLOAT(0.769103),		// 369
	UTL_SINFLOAT(0.765167),		// 370
	UTL_SINFLOAT(0.761202),		// 371
	UTL_SINFLOAT(0.757209),		// 372
	UTL_SINFLOAT(0.753187),		// 373
	UTL_SINFLOAT(0.749136),		// 374
	UTL_SINFLOAT(0.745058),		// 375
	UTL_SINFLOAT(0.740951),		// 376
	UTL_SINFLOAT(0.736817),		// 377
	UTL_SINFLOAT(0.732654),		// 378
	UTL_SINFLOAT(0.728464),		// 379
	UTL_SINFLOAT(0.724247),		// 380
	UTL_SINFLOAT(0.720003),		// 381
	UTL_SINFLOAT(0.715731),		// 382
	UTL_SINFLOAT(0.711432),		// 383
	UTL_SINFLOAT(0.707107),		// 384
	UTL_SINFLOAT(0.702755),		// 385
	UTL_SINFLOAT(0.698376),		// 386
	UTL_SINFLOAT(0.693971),		// 387
	UTL_SINFLOAT(0.689541),		// 388
	UTL_SINFLOAT(0.685084),		// 389
	UTL_SINFLOAT(0.680601),		// 390
	UTL_SINFLOAT(0.676093),		// 391
	UTL_SINFLOAT(0.671559),		// 392
	UTL_SINFLOAT(0.667000),		// 393
	UTL_SINFLOAT(0.662416),		// 394
	UTL_SINFLOAT(0.657807),		// 395
	UTL_SINFLOAT(0.653173),		// 396
	UTL_SINFLOAT(0.648514),		// 397
	UTL_SINFLOAT(0.643832),		// 398
	UTL_SINFLOAT(0.639124),		// 399
	UTL_SINFLOAT(0.634393),		// 400
	UTL_SINFLOAT(0.629638),		// 401
	UTL_SINFLOAT(0.624859),		// 402
	UTL_SINFLOAT(0.620057),		// 403
	UTL_SINFLOAT(0.615232),		// 404
	UTL_SINFLOAT(0.610383),		// 405
	UTL_SINFLOAT(0.605511),		// 406
	UTL_SINFLOAT(0.600616),		// 407
	UTL_SINFLOAT(0.595699),		// 408
	UTL_SINFLOAT(0.590760),		// 409
	UTL_SINFLOAT(0.585798),		// 410
	UTL_SINFLOAT(0.580814),		// 411
	UTL_SINFLOAT(0.575808),		// 412
	UTL_SINFLOAT(0.570781),		// 413
	UTL_SINFLOAT(0.565732),		// 414
	UTL_SINFLOAT(0.560662),		// 415
	UTL_SINFLOAT(0.555570),		// 416
	UTL_SINFLOAT(0.550458),		// 417
	UTL_SINFLOAT(0.545325),		// 418
	UTL_SINFLOAT(0.540171),		// 419
	UTL_SINFLOAT(0.534998),		// 420
	UTL_SINFLOAT(0.529804),		// 421
	UTL_SINFLOAT(0.524590),		// 422
	UTL_SINFLOAT(0.519356),		// 423
	UTL_SINFLOAT(0.514103),		// 424
	UTL_SINFLOAT(0.508830),		// 425
	UTL_SINFLOAT(0.503538),		// 426
	UTL_SINFLOAT(0.498228),		// 427
	UTL_SINFLOAT(0.492898),		// 428
	UTL_SINFLOAT(0.487550),		// 429
	UTL_SINFLOAT(0.482184),		// 430
	UTL_SINFLOAT(0.476799),		// 431
	UTL_SINFLOAT(0.471397),		// 432
	UTL_SINFLOAT(0.465976),		// 433
	UTL_SINFLOAT(0.460539),		// 434
	UTL_SINFLOAT(0.455084),		// 435
	UTL_SINFLOAT(0.449611),		// 436
	UTL_SINFLOAT(0.444122),		// 437
	UTL_SINFLOAT(0.438616),		// 438
	UTL_SINFLOAT(0.433094),		// 439
	UTL_SINFLOAT(0.427555),		// 440
	UTL_SINFLOAT(0.422000),		// 441
	UTL_SINFLOAT(0.416430),		// 442
	UTL_SINFLOAT(0.410843),		// 443
	UTL_SINFLOAT(0.405241),		// 444
	UTL_SINFLOAT(0.399624),		// 445
	UTL_SINFLOAT(0.393992),		// 446
	UTL_SINFLOAT(0.388345),		// 447
	UTL_SINFLOAT(0.382683),		// 448
	UTL_SINFLOAT(0.377007),		// 449
	UTL_SINFLOAT(0.371317),		// 450
	UTL_SINFLOAT(0.365613),		// 451
	UTL_SINFLOAT(0.359895),		// 452
	UTL_SINFLOAT(0.354164),		// 453
	UTL_SINFLOAT(0.348419),		// 454
	UTL_SINFLOAT(0.342661),		// 455
	UTL_SINFLOAT(0.336890),		// 456
	UTL_SINFLOAT(0.331106),		// 457
	UTL_SINFLOAT(0.325310),		// 458
	UTL_SINFLOAT(0.319502),		// 459
	UTL_SINFLOAT(0.313682),		// 460
	UTL_SINFLOAT(0.307850),		// 461
	UTL_SINFLOAT(0.302006),		// 462
	UTL_SINFLOAT(0.296151),		// 463
	UTL_SINFLOAT(0.290285),		// 464
	UTL_SINFLOAT(0.284408),		// 465
	UTL_SINFLOAT(0.278520),		// 466
	UTL_SINFLOAT(0.272621),		// 467
	UTL_SINFLOAT(0.266713),		// 468
	UTL_SINFLOAT(0.260794),		// 469
	UTL_SINFLOAT(0.254866),		// 470
	UTL_SINFLOAT(0.248928),		// 471
	UTL_SINFLOAT(0.242980),		// 472
	UTL_SINFLOAT(0.237024),		// 473
	UTL_SINFLOAT(0.231058),		// 474
	UTL_SINFLOAT(0.225084),		// 475
	UTL_SINFLOAT(0.219101),		// 476
	UTL_SINFLOAT(0.213110),		// 477
	UTL_SINFLOAT(0.207111),		// 478
	UTL_SINFLOAT(0.201105),		// 479
	UTL_SINFLOAT(0.195090),		// 480
	UTL_SINFLOAT(0.189069),		// 481
	UTL_SINFLOAT(0.183040),		// 482
	UTL_SINFLOAT(0.177004),		// 483
	UTL_SINFLOAT(0.170962),		// 484
	UTL_SINFLOAT(0.164913),		// 485
	UTL_SINFLOAT(0.158858),		// 486
	UTL_SINFLOAT(0.152797),		// 487
	UTL_SINFLOAT(0.146730),		// 488
	UTL_SINFLOAT(0.140658),		// 489
	UTL_SINFLOAT(0.134581),		// 490
	UTL_SINFLOAT(0.128498),		// 491
	UTL_SINFLOAT(0.122411),		// 492
	UTL_SINFLOAT(0.116319),		// 493
	UTL_SINFLOAT(0.110222),		// 494
	UTL_SINFLOAT(0.104122),		// 495
	UTL_SINFLOAT(0.098017),		// 496
	UTL_SINFLOAT(0.091909),		// 497
	UTL_SINFLOAT(0.085797),		// 498
	UTL_SINFLOAT(0.079682),		// 499
	UTL_SINFLOAT(0.073565),		// 500
	UTL_SINFLOAT(0.067444),		// 501
	UTL_SINFLOAT(0.061321),		// 502
	UTL_SINFLOAT(0.055195),		// 503
	UTL_SINFLOAT(0.049068),		// 504
	UTL_SINFLOAT(0.042938),		// 505
	UTL_SINFLOAT(0.036807),		// 506
	UTL_SINFLOAT(0.030675),		// 507
	UTL_SINFLOAT(0.024541),		// 508
	UTL_SINFLOAT(0.018407),		// 509
	UTL_SINFLOAT(0.012272),		// 510
	UTL_SINFLOAT(0.006136),		// 511
	UTL_SINFLOAT(0.000000),		// 512
	UTL_SINFLOAT(-0.006136),	// 513
	UTL_SINFLOAT(-0.012272),	// 514
	UTL_SINFLOAT(-0.018407),	// 515
	UTL_SINFLOAT(-0.024541),	// 516
	UTL_SINFLOAT(-0.030675),	// 517
	UTL_SINFLOAT(-0.036807),	// 518
	UTL_SINFLOAT(-0.042938),	// 519
	UTL_SINFLOAT(-0.049068),	// 520
	UTL_SINFLOAT(-0.055195),	// 521
	UTL_SINFLOAT(-0.061321),	// 522
	UTL_SINFLOAT(-0.067444),	// 523
	UTL_SINFLOAT(-0.073565),	// 524
	UTL_SINFLOAT(-0.079682),	// 525
	UTL_SINFLOAT(-0.085797),	// 526
	UTL_SINFLOAT(-0.091909),	// 527
	UTL_SINFLOAT(-0.098017),	// 528
	UTL_SINFLOAT(-0.104122),	// 529
	UTL_SINFLOAT(-0.110222),	// 530
	UTL_SINFLOAT(-0.116319),	// 531
	UTL_SINFLOAT(-0.122411),	// 532
	UTL_SINFLOAT(-0.128498),	// 533
	UTL_SINFLOAT(-0.134581),	// 534
	UTL_SINFLOAT(-0.140658),	// 535
	UTL_SINFLOAT(-0.146730),	// 536
	UTL_SINFLOAT(-0.152797),	// 537
	UTL_SINFLOAT(-0.158858),	// 538
	UTL_SINFLOAT(-0.164913),	// 539
	UTL_SINFLOAT(-0.170962),	// 540
	UTL_SINFLOAT(-0.177004),	// 541
	UTL_SINFLOAT(-0.183040),	// 542
	UTL_SINFLOAT(-0.189069),	// 543
	UTL_SINFLOAT(-0.195090),	// 544
	UTL_SINFLOAT(-0.201105),	// 545
	UTL_SINFLOAT(-0.207111),	// 546
	UTL_SINFLOAT(-0.213110),	// 547
	UTL_SINFLOAT(-0.219101),	// 548
	UTL_SINFLOAT(-0.225084),	// 549
	UTL_SINFLOAT(-0.231058),	// 550
	UTL_SINFLOAT(-0.237024),	// 551
	UTL_SINFLOAT(-0.242980),	// 552
	UTL_SINFLOAT(-0.248928),	// 553
	UTL_SINFLOAT(-0.254866),	// 554
	UTL_SINFLOAT(-0.260794),	// 555
	UTL_SINFLOAT(-0.266713),	// 556
	UTL_SINFLOAT(-0.272621),	// 557
	UTL_SINFLOAT(-0.278520),	// 558
	UTL_SINFLOAT(-0.284408),	// 559
	UTL_SINFLOAT(-0.290285),	// 560
	UTL_SINFLOAT(-0.296151),	// 561
	UTL_SINFLOAT(-0.302006),	// 562
	UTL_SINFLOAT(-0.307850),	// 563
	UTL_SINFLOAT(-0.313682),	// 564
	UTL_SINFLOAT(-0.319502),	// 565
	UTL_SINFLOAT(-0.325310),	// 566
	UTL_SINFLOAT(-0.331106),	// 567
	UTL_SINFLOAT(-0.336890),	// 568
	UTL_SINFLOAT(-0.342661),	// 569
	UTL_SINFLOAT(-0.348419),	// 570
	UTL_SINFLOAT(-0.354164),	// 571
	UTL_SINFLOAT(-0.359895),	// 572
	UTL_SINFLOAT(-0.365613),	// 573
	UTL_SINFLOAT(-0.371317),	// 574
	UTL_SINFLOAT(-0.377007),	// 575
	UTL_SINFLOAT(-0.382683),	// 576
	UTL_SINFLOAT(-0.388345),	// 577
	UTL_SINFLOAT(-0.393992),	// 578
	UTL_SINFLOAT(-0.399624),	// 579
	UTL_SINFLOAT(-0.405241),	// 580
	UTL_SINFLOAT(-0.410843),	// 581
	UTL_SINFLOAT(-0.416430),	// 582
	UTL_SINFLOAT(-0.422000),	// 583
	UTL_SINFLOAT(-0.427555),	// 584
	UTL_SINFLOAT(-0.433094),	// 585
	UTL_SINFLOAT(-0.438616),	// 586
	UTL_SINFLOAT(-0.444122),	// 587
	UTL_SINFLOAT(-0.449611),	// 588
	UTL_SINFLOAT(-0.455084),	// 589
	UTL_SINFLOAT(-0.460539),	// 590
	UTL_SINFLOAT(-0.465976),	// 591
	UTL_SINFLOAT(-0.471397),	// 592
	UTL_SINFLOAT(-0.476799),	// 593
	UTL_SINFLOAT(-0.482184),	// 594
	UTL_SINFLOAT(-0.487550),	// 595
	UTL_SINFLOAT(-0.492898),	// 596
	UTL_SINFLOAT(-0.498228),	// 597
	UTL_SINFLOAT(-0.503538),	// 598
	UTL_SINFLOAT(-0.508830),	// 599
	UTL_SINFLOAT(-0.514103),	// 600
	UTL_SINFLOAT(-0.519356),	// 601
	UTL_SINFLOAT(-0.524590),	// 602
	UTL_SINFLOAT(-0.529804),	// 603
	UTL_SINFLOAT(-0.534998),	// 604
	UTL_SINFLOAT(-0.540171),	// 605
	UTL_SINFLOAT(-0.545325),	// 606
	UTL_SINFLOAT(-0.550458),	// 607
	UTL_SINFLOAT(-0.555570),	// 608
	UTL_SINFLOAT(-0.560662),	// 609
	UTL_SINFLOAT(-0.565732),	// 610
	UTL_SINFLOAT(-0.570781),	// 611
	UTL_SINFLOAT(-0.575808),	// 612
	UTL_SINFLOAT(-0.580814),	// 613
	UTL_SINFLOAT(-0.585798),	// 614
	UTL_SINFLOAT(-0.590760),	// 615
	UTL_SINFLOAT(-0.595699),	// 616
	UTL_SINFLOAT(-0.600616),	// 617
	UTL_SINFLOAT(-0.605511),	// 618
	UTL_SINFLOAT(-0.610383),	// 619
	UTL_SINFLOAT(-0.615232),	// 620
	UTL_SINFLOAT(-0.620057),	// 621
	UTL_SINFLOAT(-0.624859),	// 622
	UTL_SINFLOAT(-0.629638),	// 623
	UTL_SINFLOAT(-0.634393),	// 624
	UTL_SINFLOAT(-0.639124),	// 625
	UTL_SINFLOAT(-0.643832),	// 626
	UTL_SINFLOAT(-0.648514),	// 627
	UTL_SINFLOAT(-0.653173),	// 628
	UTL_SINFLOAT(-0.657807),	// 629
	UTL_SINFLOAT(-0.662416),	// 630
	UTL_SINFLOAT(-0.667000),	// 631
	UTL_SINFLOAT(-0.671559),	// 632
	UTL_SINFLOAT(-0.676093),	// 633
	UTL_SINFLOAT(-0.680601),	// 634
	UTL_SINFLOAT(-0.685084),	// 635
	UTL_SINFLOAT(-0.689541),	// 636
	UTL_SINFLOAT(-0.693971),	// 637
	UTL_SINFLOAT(-0.698376),	// 638
	UTL_SINFLOAT(-0.702755),	// 639
	UTL_SINFLOAT(-0.707107),	// 640
	UTL_SINFLOAT(-0.711432),	// 641
	UTL_SINFLOAT(-0.715731),	// 642
	UTL_SINFLOAT(-0.720003),	// 643
	UTL_SINFLOAT(-0.724247),	// 644
	UTL_SINFLOAT(-0.728464),	// 645
	UTL_SINFLOAT(-0.732654),	// 646
	UTL_SINFLOAT(-0.736817),	// 647
	UTL_SINFLOAT(-0.740951),	// 648
	UTL_SINFLOAT(-0.745058),	// 649
	UTL_SINFLOAT(-0.749136),	// 650
	UTL_SINFLOAT(-0.753187),	// 651
	UTL_SINFLOAT(-0.757209),	// 652
	UTL_SINFLOAT(-0.761202),	// 653
	UTL_SINFLOAT(-0.765167),	// 654
	UTL_SINFLOAT(-0.769103),	// 655
	UTL_SINFLOAT(-0.773010),	// 656
	UTL_SINFLOAT(-0.776888),	// 657
	UTL_SINFLOAT(-0.780737),	// 658
	UTL_SINFLOAT(-0.784557),	// 659
	UTL_SINFLOAT(-0.788346),	// 660
	UTL_SINFLOAT(-0.792107),	// 661
	UTL_SINFLOAT(-0.795837),	// 662
	UTL_SINFLOAT(-0.799537),	// 663
	UTL_SINFLOAT(-0.803208),	// 664
	UTL_SINFLOAT(-0.806848),	// 665
	UTL_SINFLOAT(-0.810457),	// 666
	UTL_SINFLOAT(-0.814036),	// 667
	UTL_SINFLOAT(-0.817585),	// 668
	UTL_SINFLOAT(-0.821103),	// 669
	UTL_SINFLOAT(-0.824589),	// 670
	UTL_SINFLOAT(-0.828045),	// 671
	UTL_SINFLOAT(-0.831470),	// 672
	UTL_SINFLOAT(-0.834863),	// 673
	UTL_SINFLOAT(-0.838225),	// 674
	UTL_SINFLOAT(-0.841555),	// 675
	UTL_SINFLOAT(-0.844854),	// 676
	UTL_SINFLOAT(-0.848120),	// 677
	UTL_SINFLOAT(-0.851355),	// 678
	UTL_SINFLOAT(-0.854558),	// 679
	UTL_SINFLOAT(-0.857729),	// 680
	UTL_SINFLOAT(-0.860867),	// 681
	UTL_SINFLOAT(-0.863973),	// 682
	UTL_SINFLOAT(-0.867046),	// 683
	UTL_SINFLOAT(-0.870087),	// 684
	UTL_SINFLOAT(-0.873095),	// 685
	UTL_SINFLOAT(-0.876070),	// 686
	UTL_SINFLOAT(-0.879012),	// 687
	UTL_SINFLOAT(-0.881921),	// 688
	UTL_SINFLOAT(-0.884797),	// 689
	UTL_SINFLOAT(-0.887640),	// 690
	UTL_SINFLOAT(-0.890449),	// 691
	UTL_SINFLOAT(-0.893224),	// 692
	UTL_SINFLOAT(-0.895966),	// 693
	UTL_SINFLOAT(-0.898674),	// 694
	UTL_SINFLOAT(-0.901349),	// 695
	UTL_SINFLOAT(-0.903989),	// 696
	UTL_SINFLOAT(-0.906596),	// 697
	UTL_SINFLOAT(-0.909168),	// 698
	UTL_SINFLOAT(-0.911706),	// 699
	UTL_SINFLOAT(-0.914210),	// 700
	UTL_SINFLOAT(-0.916679),	// 701
	UTL_SINFLOAT(-0.919114),	// 702
	UTL_SINFLOAT(-0.921514),	// 703
	UTL_SINFLOAT(-0.923880),	// 704
	UTL_SINFLOAT(-0.926210),	// 705
	UTL_SINFLOAT(-0.928506),	// 706
	UTL_SINFLOAT(-0.930767),	// 707
	UTL_SINFLOAT(-0.932993),	// 708
	UTL_SINFLOAT(-0.935184),	// 709
	UTL_SINFLOAT(-0.937339),	// 710
	UTL_SINFLOAT(-0.939459),	// 711
	UTL_SINFLOAT(-0.941544),	// 712
	UTL_SINFLOAT(-0.943593),	// 713
	UTL_SINFLOAT(-0.945607),	// 714
	UTL_SINFLOAT(-0.947586),	// 715
	UTL_SINFLOAT(-0.949528),	// 716
	UTL_SINFLOAT(-0.951435),	// 717
	UTL_SINFLOAT(-0.953306),	// 718
	UTL_SINFLOAT(-0.955141),	// 719
	UTL_SINFLOAT(-0.956940),	// 720
	UTL_SINFLOAT(-0.958703),	// 721
	UTL_SINFLOAT(-0.960431),	// 722
	UTL_SINFLOAT(-0.962121),	// 723
	UTL_SINFLOAT(-0.963776),	// 724
	UTL_SINFLOAT(-0.965394),	// 725
	UTL_SINFLOAT(-0.966976),	// 726
	UTL_SINFLOAT(-0.968522),	// 727
	UTL_SINFLOAT(-0.970031),	// 728
	UTL_SINFLOAT(-0.971504),	// 729
	UTL_SINFLOAT(-0.972940),	// 730
	UTL_SINFLOAT(-0.974339),	// 731
	UTL_SINFLOAT(-0.975702),	// 732
	UTL_SINFLOAT(-0.977028),	// 733
	UTL_SINFLOAT(-0.978317),	// 734
	UTL_SINFLOAT(-0.979570),	// 735
	UTL_SINFLOAT(-0.980785),	// 736
	UTL_SINFLOAT(-0.981964),	// 737
	UTL_SINFLOAT(-0.983105),	// 738
	UTL_SINFLOAT(-0.984210),	// 739
	UTL_SINFLOAT(-0.985278),	// 740
	UTL_SINFLOAT(-0.986308),	// 741
	UTL_SINFLOAT(-0.987301),	// 742
	UTL_SINFLOAT(-0.988258),	// 743
	UTL_SINFLOAT(-0.989177),	// 744
	UTL_SINFLOAT(-0.990058),	// 745
	UTL_SINFLOAT(-0.990903),	// 746
	UTL_SINFLOAT(-0.991710),	// 747
	UTL_SINFLOAT(-0.992480),	// 748
	UTL_SINFLOAT(-0.993212),	// 749
	UTL_SINFLOAT(-0.993907),	// 750
	UTL_SINFLOAT(-0.994565),	// 751
	UTL_SINFLOAT(-0.995185),	// 752
	UTL_SINFLOAT(-0.995767),	// 753
	UTL_SINFLOAT(-0.996313),	// 754
	UTL_SINFLOAT(-0.996820),	// 755
	UTL_SINFLOAT(-0.997290),	// 756
	UTL_SINFLOAT(-0.997723),	// 757
	UTL_SINFLOAT(-0.998118),	// 758
	UTL_SINFLOAT(-0.998476),	// 759
	UTL_SINFLOAT(-0.998795),	// 760
	UTL_SINFLOAT(-0.999078),	// 761
	UTL_SINFLOAT(-0.999322),	// 762
	UTL_SINFLOAT(-0.999529),	// 763
	UTL_SINFLOAT(-0.999699),	// 764
	UTL_SINFLOAT(-0.999831),	// 765
	UTL_SINFLOAT(-0.999925),	// 766
	UTL_SINFLOAT(-0.999981),	// 767
	UTL_SINFLOAT(-1.000000),	// 768
	UTL_SINFLOAT(-0.999981),	// 769
	UTL_SINFLOAT(-0.999925),	// 770
	UTL_SINFLOAT(-0.999831),	// 771
	UTL_SINFLOAT(-0.999699),	// 772
	UTL_SINFLOAT(-0.999529),	// 773
	UTL_SINFLOAT(-0.999322),	// 774
	UTL_SINFLOAT(-0.999078),	// 775
	UTL_SINFLOAT(-0.998795),	// 776
	UTL_SINFLOAT(-0.998476),	// 777
	UTL_SINFLOAT(-0.998118),	// 778
	UTL_SINFLOAT(-0.997723),	// 779
	UTL_SINFLOAT(-0.997290),	// 780
	UTL_SINFLOAT(-0.996820),	// 781
	UTL_SINFLOAT(-0.996313),	// 782
	UTL_SINFLOAT(-0.995767),	// 783
	UTL_SINFLOAT(-0.995185),	// 784
	UTL_SINFLOAT(-0.994565),	// 785
	UTL_SINFLOAT(-0.993907),	// 786
	UTL_SINFLOAT(-0.993212),	// 787
	UTL_SINFLOAT(-0.992480),	// 788
	UTL_SINFLOAT(-0.991710),	// 789
	UTL_SINFLOAT(-0.990903),	// 790
	UTL_SINFLOAT(-0.990058),	// 791
	UTL_SINFLOAT(-0.989177),	// 792
	UTL_SINFLOAT(-0.988258),	// 793
	UTL_SINFLOAT(-0.987301),	// 794
	UTL_SINFLOAT(-0.986308),	// 795
	UTL_SINFLOAT(-0.985278),	// 796
	UTL_SINFLOAT(-0.984210),	// 797
	UTL_SINFLOAT(-0.983105),	// 798
	UTL_SINFLOAT(-0.981964),	// 799
	UTL_SINFLOAT(-0.980785),	// 800
	UTL_SINFLOAT(-0.979570),	// 801
	UTL_SINFLOAT(-0.978317),	// 802
	UTL_SINFLOAT(-0.977028),	// 803
	UTL_SINFLOAT(-0.975702),	// 804
	UTL_SINFLOAT(-0.974339),	// 805
	UTL_SINFLOAT(-0.972940),	// 806
	UTL_SINFLOAT(-0.971504),	// 807
	UTL_SINFLOAT(-0.970031),	// 808
	UTL_SINFLOAT(-0.968522),	// 809
	UTL_SINFLOAT(-0.966976),	// 810
	UTL_SINFLOAT(-0.965394),	// 811
	UTL_SINFLOAT(-0.963776),	// 812
	UTL_SINFLOAT(-0.962121),	// 813
	UTL_SINFLOAT(-0.960431),	// 814
	UTL_SINFLOAT(-0.958703),	// 815
	UTL_SINFLOAT(-0.956940),	// 816
	UTL_SINFLOAT(-0.955141),	// 817
	UTL_SINFLOAT(-0.953306),	// 818
	UTL_SINFLOAT(-0.951435),	// 819
	UTL_SINFLOAT(-0.949528),	// 820
	UTL_SINFLOAT(-0.947586),	// 821
	UTL_SINFLOAT(-0.945607),	// 822
	UTL_SINFLOAT(-0.943593),	// 823
	UTL_SINFLOAT(-0.941544),	// 824
	UTL_SINFLOAT(-0.939459),	// 825
	UTL_SINFLOAT(-0.937339),	// 826
	UTL_SINFLOAT(-0.935184),	// 827
	UTL_SINFLOAT(-0.932993),	// 828
	UTL_SINFLOAT(-0.930767),	// 829
	UTL_SINFLOAT(-0.928506),	// 830
	UTL_SINFLOAT(-0.926210),	// 831
	UTL_SINFLOAT(-0.923880),	// 832
	UTL_SINFLOAT(-0.921514),	// 833
	UTL_SINFLOAT(-0.919114),	// 834
	UTL_SINFLOAT(-0.916679),	// 835
	UTL_SINFLOAT(-0.914210),	// 836
	UTL_SINFLOAT(-0.911706),	// 837
	UTL_SINFLOAT(-0.909168),	// 838
	UTL_SINFLOAT(-0.906596),	// 839
	UTL_SINFLOAT(-0.903989),	// 840
	UTL_SINFLOAT(-0.901349),	// 841
	UTL_SINFLOAT(-0.898674),	// 842
	UTL_SINFLOAT(-0.895966),	// 843
	UTL_SINFLOAT(-0.893224),	// 844
	UTL_SINFLOAT(-0.890449),	// 845
	UTL_SINFLOAT(-0.887640),	// 846
	UTL_SINFLOAT(-0.884797),	// 847
	UTL_SINFLOAT(-0.881921),	// 848
	UTL_SINFLOAT(-0.879012),	// 849
	UTL_SINFLOAT(-0.876070),	// 850
	UTL_SINFLOAT(-0.873095),	// 851
	UTL_SINFLOAT(-0.870087),	// 852
	UTL_SINFLOAT(-0.867046),	// 853
	UTL_SINFLOAT(-0.863973),	// 854
	UTL_SINFLOAT(-0.860867),	// 855
	UTL_SINFLOAT(-0.857729),	// 856
	UTL_SINFLOAT(-0.854558),	// 857
	UTL_SINFLOAT(-0.851355),	// 858
	UTL_SINFLOAT(-0.848120),	// 859
	UTL_SINFLOAT(-0.844854),	// 860
	UTL_SINFLOAT(-0.841555),	// 861
	UTL_SINFLOAT(-0.838225),	// 862
	UTL_SINFLOAT(-0.834863),	// 863
	UTL_SINFLOAT(-0.831470),	// 864
	UTL_SINFLOAT(-0.828045),	// 865
	UTL_SINFLOAT(-0.824589),	// 866
	UTL_SINFLOAT(-0.821103),	// 867
	UTL_SINFLOAT(-0.817585),	// 868
	UTL_SINFLOAT(-0.814036),	// 869
	UTL_SINFLOAT(-0.810457),	// 870
	UTL_SINFLOAT(-0.806848),	// 871
	UTL_SINFLOAT(-0.803208),	// 872
	UTL_SINFLOAT(-0.799537),	// 873
	UTL_SINFLOAT(-0.795837),	// 874
	UTL_SINFLOAT(-0.792107),	// 875
	UTL_SINFLOAT(-0.788346),	// 876
	UTL_SINFLOAT(-0.784557),	// 877
	UTL_SINFLOAT(-0.780737),	// 878
	UTL_SINFLOAT(-0.776888),	// 879
	UTL_SINFLOAT(-0.773010),	// 880
	UTL_SINFLOAT(-0.769103),	// 881
	UTL_SINFLOAT(-0.765167),	// 882
	UTL_SINFLOAT(-0.761202),	// 883
	UTL_SINFLOAT(-0.757209),	// 884
	UTL_SINFLOAT(-0.753187),	// 885
	UTL_SINFLOAT(-0.749136),	// 886
	UTL_SINFLOAT(-0.745058),	// 887
	UTL_SINFLOAT(-0.740951),	// 888
	UTL_SINFLOAT(-0.736817),	// 889
	UTL_SINFLOAT(-0.732654),	// 890
	UTL_SINFLOAT(-0.728464),	// 891
	UTL_SINFLOAT(-0.724247),	// 892
	UTL_SINFLOAT(-0.720003),	// 893
	UTL_SINFLOAT(-0.715731),	// 894
	UTL_SINFLOAT(-0.711432),	// 895
	UTL_SINFLOAT(-0.707107),	// 896
	UTL_SINFLOAT(-0.702755),	// 897
	UTL_SINFLOAT(-0.698376),	// 898
	UTL_SINFLOAT(-0.693971),	// 899
	UTL_SINFLOAT(-0.689541),	// 900
	UTL_SINFLOAT(-0.685084),	// 901
	UTL_SINFLOAT(-0.680601),	// 902
	UTL_SINFLOAT(-0.676093),	// 903
	UTL_SINFLOAT(-0.671559),	// 904
	UTL_SINFLOAT(-0.667000),	// 905
	UTL_SINFLOAT(-0.662416),	// 906
	UTL_SINFLOAT(-0.657807),	// 907
	UTL_SINFLOAT(-0.653173),	// 908
	UTL_SINFLOAT(-0.648514),	// 909
	UTL_SINFLOAT(-0.643832),	// 910
	UTL_SINFLOAT(-0.639124),	// 911
	UTL_SINFLOAT(-0.634393),	// 912
	UTL_SINFLOAT(-0.629638),	// 913
	UTL_SINFLOAT(-0.624859),	// 914
	UTL_SINFLOAT(-0.620057),	// 915
	UTL_SINFLOAT(-0.615232),	// 916
	UTL_SINFLOAT(-0.610383),	// 917
	UTL_SINFLOAT(-0.605511),	// 918
	UTL_SINFLOAT(-0.600616),	// 919
	UTL_SINFLOAT(-0.595699),	// 920
	UTL_SINFLOAT(-0.590760),	// 921
	UTL_SINFLOAT(-0.585798),	// 922
	UTL_SINFLOAT(-0.580814),	// 923
	UTL_SINFLOAT(-0.575808),	// 924
	UTL_SINFLOAT(-0.570781),	// 925
	UTL_SINFLOAT(-0.565732),	// 926
	UTL_SINFLOAT(-0.560662),	// 927
	UTL_SINFLOAT(-0.555570),	// 928
	UTL_SINFLOAT(-0.550458),	// 929
	UTL_SINFLOAT(-0.545325),	// 930
	UTL_SINFLOAT(-0.540171),	// 931
	UTL_SINFLOAT(-0.534998),	// 932
	UTL_SINFLOAT(-0.529804),	// 933
	UTL_SINFLOAT(-0.524590),	// 934
	UTL_SINFLOAT(-0.519356),	// 935
	UTL_SINFLOAT(-0.514103),	// 936
	UTL_SINFLOAT(-0.508830),	// 937
	UTL_SINFLOAT(-0.503538),	// 938
	UTL_SINFLOAT(-0.498228),	// 939
	UTL_SINFLOAT(-0.492898),	// 940
	UTL_SINFLOAT(-0.487550),	// 941
	UTL_SINFLOAT(-0.482184),	// 942
	UTL_SINFLOAT(-0.476799),	// 943
	UTL_SINFLOAT(-0.471397),	// 944
	UTL_SINFLOAT(-0.465976),	// 945
	UTL_SINFLOAT(-0.460539),	// 946
	UTL_SINFLOAT(-0.455084),	// 947
	UTL_SINFLOAT(-0.449611),	// 948
	UTL_SINFLOAT(-0.444122),	// 949
	UTL_SINFLOAT(-0.438616),	// 950
	UTL_SINFLOAT(-0.433094),	// 951
	UTL_SINFLOAT(-0.427555),	// 952
	UTL_SINFLOAT(-0.422000),	// 953
	UTL_SINFLOAT(-0.416430),	// 954
	UTL_SINFLOAT(-0.410843),	// 955
	UTL_SINFLOAT(-0.405241),	// 956
	UTL_SINFLOAT(-0.399624),	// 957
	UTL_SINFLOAT(-0.393992),	// 958
	UTL_SINFLOAT(-0.388345),	// 959
	UTL_SINFLOAT(-0.382683),	// 960
	UTL_SINFLOAT(-0.377007),	// 961
	UTL_SINFLOAT(-0.371317),	// 962
	UTL_SINFLOAT(-0.365613),	// 963
	UTL_SINFLOAT(-0.359895),	// 964
	UTL_SINFLOAT(-0.354164),	// 965
	UTL_SINFLOAT(-0.348419),	// 966
	UTL_SINFLOAT(-0.342661),	// 967
	UTL_SINFLOAT(-0.336890),	// 968
	UTL_SINFLOAT(-0.331106),	// 969
	UTL_SINFLOAT(-0.325310),	// 970
	UTL_SINFLOAT(-0.319502),	// 971
	UTL_SINFLOAT(-0.313682),	// 972
	UTL_SINFLOAT(-0.307850),	// 973
	UTL_SINFLOAT(-0.302006),	// 974
	UTL_SINFLOAT(-0.296151),	// 975
	UTL_SINFLOAT(-0.290285),	// 976
	UTL_SINFLOAT(-0.284408),	// 977
	UTL_SINFLOAT(-0.278520),	// 978
	UTL_SINFLOAT(-0.272621),	// 979
	UTL_SINFLOAT(-0.266713),	// 980
	UTL_SINFLOAT(-0.260794),	// 981
	UTL_SINFLOAT(-0.254866),	// 982
	UTL_SINFLOAT(-0.248928),	// 983
	UTL_SINFLOAT(-0.242980),	// 984
	UTL_SINFLOAT(-0.237024),	// 985
	UTL_SINFLOAT(-0.231058),	// 986
	UTL_SINFLOAT(-0.225084),	// 987
	UTL_SINFLOAT(-0.219101),	// 988
	UTL_SINFLOAT(-0.213110),	// 989
	UTL_SINFLOAT(-0.207111),	// 990
	UTL_SINFLOAT(-0.201105),	// 991
	UTL_SINFLOAT(-0.195090),	// 992
	UTL_SINFLOAT(-0.189069),	// 993
	UTL_SINFLOAT(-0.183040),	// 994
	UTL_SINFLOAT(-0.177004),	// 995
	UTL_SINFLOAT(-0.170962),	// 996
	UTL_SINFLOAT(-0.164913),	// 997
	UTL_SINFLOAT(-0.158858),	// 998
	UTL_SINFLOAT(-0.152797),	// 999
	UTL_SINFLOAT(-0.146730),	// 1000
	UTL_SINFLOAT(-0.140658),	// 1001
	UTL_SINFLOAT(-0.134581),	// 1002
	UTL_SINFLOAT(-0.128498),	// 1003
	UTL_SINFLOAT(-0.122411),	// 1004
	UTL_SINFLOAT(-0.116319),	// 1005
	UTL_SINFLOAT(-0.110222),	// 1006
	UTL_SINFLOAT(-0.104122),	// 1007
	UTL_SINFLOAT(-0.098017),	// 1008
	UTL_SINFLOAT(-0.091909),	// 1009
	UTL_SINFLOAT(-0.085797),	// 1010
	UTL_SINFLOAT(-0.079682),	// 1011
	UTL_SINFLOAT(-0.073565),	// 1012
	UTL_SINFLOAT(-0.067444),	// 1013
	UTL_SINFLOAT(-0.061321),	// 1014
	UTL_SINFLOAT(-0.055195),	// 1015
	UTL_SINFLOAT(-0.049068),	// 1016
	UTL_SINFLOAT(-0.042938),	// 1017
	UTL_SINFLOAT(-0.036807),	// 1018
	UTL_SINFLOAT(-0.030675),	// 1019
	UTL_SINFLOAT(-0.024541),	// 1020
	UTL_SINFLOAT(-0.018407),	// 1021
	UTL_SINFLOAT(-0.012272),	// 1022
	UTL_SINFLOAT(-0.006136),	// 1023
};

int32 SinTable1024[] =
{
	0x00000000L,
	0x00000064L,
	0x000000C9L,
	0x0000012DL,
	0x00000192L,
	0x000001F6L,
	0x0000025BL,
	0x000002BFL,
	0x00000324L,
	0x00000388L,
	0x000003EDL,
	0x00000451L,
	0x000004B6L,
	0x0000051AL,
	0x0000057FL,
	0x000005E3L,
	0x00000648L,
	0x000006ACL,
	0x00000711L,
	0x00000775L,
	0x000007DAL,
	0x0000083EL,
	0x000008A3L,
	0x00000907L,
	0x0000096CL,
	0x000009D0L,
	0x00000A35L,
	0x00000A99L,
	0x00000AFEL,
	0x00000B62L,
	0x00000BC6L,
	0x00000C2BL,
	0x00000C8FL,
	0x00000CF4L,
	0x00000D58L,
	0x00000DBCL,
	0x00000E21L,
	0x00000E85L,
	0x00000EEAL,
	0x00000F4EL,
	0x00000FB2L,
	0x00001017L,
	0x0000107BL,
	0x000010DFL,
	0x00001144L,
	0x000011A8L,
	0x0000120CL,
	0x00001270L,
	0x000012D5L,
	0x00001339L,
	0x0000139DL,
	0x00001401L,
	0x00001466L,
	0x000014CAL,
	0x0000152EL,
	0x00001592L,
	0x000015F6L,
	0x0000165AL,
	0x000016BFL,
	0x00001723L,
	0x00001787L,
	0x000017EBL,
	0x0000184FL,
	0x000018B3L,
	0x00001917L,
	0x0000197BL,
	0x000019DFL,
	0x00001A43L,
	0x00001AA7L,
	0x00001B0BL,
	0x00001B6FL,
	0x00001BD3L,
	0x00001C37L,
	0x00001C9BL,
	0x00001CFFL,
	0x00001D63L,
	0x00001DC7L,
	0x00001E2AL,
	0x00001E8EL,
	0x00001EF2L,
	0x00001F56L,
	0x00001FBAL,
	0x0000201DL,
	0x00002081L,
	0x000020E5L,
	0x00002148L,
	0x000021ACL,
	0x00002210L,
	0x00002273L,
	0x000022D7L,
	0x0000233BL,
	0x0000239EL,
	0x00002402L,
	0x00002465L,
	0x000024C9L,
	0x0000252CL,
	0x00002590L,
	0x000025F3L,
	0x00002656L,
	0x000026BAL,
	0x0000271DL,
	0x00002781L,
	0x000027E4L,
	0x00002847L,
	0x000028AAL,
	0x0000290EL,
	0x00002971L,
	0x000029D4L,
	0x00002A37L,
	0x00002A9AL,
	0x00002AFEL,
	0x00002B61L,
	0x00002BC4L,
	0x00002C27L,
	0x00002C8AL,
	0x00002CEDL,
	0x00002D50L,
	0x00002DB3L,
	0x00002E15L,
	0x00002E78L,
	0x00002EDBL,
	0x00002F3EL,
	0x00002FA1L,
	0x00003004L,
	0x00003066L,
	0x000030C9L,
	0x0000312CL,
	0x0000318EL,
	0x000031F1L,
	0x00003254L,
	0x000032B6L,
	0x00003319L,
	0x0000337BL,
	0x000033DEL,
	0x00003440L,
	0x000034A2L,
	0x00003505L,
	0x00003567L,
	0x000035C9L,
	0x0000362CL,
	0x0000368EL,
	0x000036F0L,
	0x00003752L,
	0x000037B4L,
	0x00003817L,
	0x00003879L,
	0x000038DBL,
	0x0000393DL,
	0x0000399FL,
	0x00003A01L,
	0x00003A62L,
	0x00003AC4L,
	0x00003B26L,
	0x00003B88L,
	0x00003BEAL,
	0x00003C4BL,
	0x00003CADL,
	0x00003D0FL,
	0x00003D70L,
	0x00003DD2L,
	0x00003E33L,
	0x00003E95L,
	0x00003EF6L,
	0x00003F58L,
	0x00003FB9L,
	0x0000401BL,
	0x0000407CL,
	0x000040DDL,
	0x0000413EL,
	0x000041A0L,
	0x00004201L,
	0x00004262L,
	0x000042C3L,
	0x00004324L,
	0x00004385L,
	0x000043E6L,
	0x00004447L,
	0x000044A8L,
	0x00004508L,
	0x00004569L,
	0x000045CAL,
	0x0000462BL,
	0x0000468BL,
	0x000046ECL,
	0x0000474DL,
	0x000047ADL,
	0x0000480EL,
	0x0000486EL,
	0x000048CEL,
	0x0000492FL,
	0x0000498FL,
	0x000049EFL,
	0x00004A50L,
	0x00004AB0L,
	0x00004B10L,
	0x00004B70L,
	0x00004BD0L,
	0x00004C30L,
	0x00004C90L,
	0x00004CF0L,
	0x00004D50L,
	0x00004DB0L,
	0x00004E0FL,
	0x00004E6FL,
	0x00004ECFL,
	0x00004F2EL,
	0x00004F8EL,
	0x00004FEDL,
	0x0000504DL,
	0x000050ACL,
	0x0000510CL,
	0x0000516BL,
	0x000051CAL,
	0x0000522AL,
	0x00005289L,
	0x000052E8L,
	0x00005347L,
	0x000053A6L,
	0x00005405L,
	0x00005464L,
	0x000054C3L,
	0x00005522L,
	0x00005581L,
	0x000055DFL,
	0x0000563EL,
	0x0000569DL,
	0x000056FBL,
	0x0000575AL,
	0x000057B8L,
	0x00005817L,
	0x00005875L,
	0x000058D3L,
	0x00005931L,
	0x00005990L,
	0x000059EEL,
	0x00005A4CL,
	0x00005AAAL,
	0x00005B08L,
	0x00005B66L,
	0x00005BC4L,
	0x00005C22L,
	0x00005C7FL,
	0x00005CDDL,
	0x00005D3BL,
	0x00005D98L,
	0x00005DF6L,
	0x00005E53L,
	0x00005EB1L,
	0x00005F0EL,
	0x00005F6BL,
	0x00005FC9L,
	0x00006026L,
	0x00006083L,
	0x000060E0L,
	0x0000613DL,
	0x0000619AL,
	0x000061F7L,
	0x00006254L,
	0x000062B1L,
	0x0000630DL,
	0x0000636AL,
	0x000063C7L,
	0x00006423L,
	0x00006480L,
	0x000064DCL,
	0x00006539L,
	0x00006595L,
	0x000065F1L,
	0x0000664DL,
	0x000066A9L,
	0x00006705L,
	0x00006761L,
	0x000067BDL,
	0x00006819L,
	0x00006875L,
	0x000068D1L,
	0x0000692DL,
	0x00006988L,
	0x000069E4L,
	0x00006A3FL,
	0x00006A9BL,
	0x00006AF6L,
	0x00006B51L,
	0x00006BADL,
	0x00006C08L,
	0x00006C63L,
	0x00006CBEL,
	0x00006D19L,
	0x00006D74L,
	0x00006DCFL,
	0x00006E29L,
	0x00006E84L,
	0x00006EDFL,
	0x00006F39L,
	0x00006F94L,
	0x00006FEEL,
	0x00007049L,
	0x000070A3L,
	0x000070FDL,
	0x00007157L,
	0x000071B1L,
	0x0000720CL,
	0x00007265L,
	0x000072BFL,
	0x00007319L,
	0x00007373L,
	0x000073CDL,
	0x00007426L,
	0x00007480L,
	0x000074D9L,
	0x00007533L,
	0x0000758CL,
	0x000075E5L,
	0x0000763FL,
	0x00007698L,
	0x000076F1L,
	0x0000774AL,
	0x000077A3L,
	0x000077FBL,
	0x00007854L,
	0x000078ADL,
	0x00007906L,
	0x0000795EL,
	0x000079B7L,
	0x00007A0FL,
	0x00007A67L,
	0x00007AC0L,
	0x00007B18L,
	0x00007B70L,
	0x00007BC8L,
	0x00007C20L,
	0x00007C78L,
	0x00007CD0L,
	0x00007D27L,
	0x00007D7FL,
	0x00007DD7L,
	0x00007E2EL,
	0x00007E86L,
	0x00007EDDL,
	0x00007F34L,
	0x00007F8BL,
	0x00007FE2L,
	0x0000803AL,
	0x00008090L,
	0x000080E7L,
	0x0000813EL,
	0x00008195L,
	0x000081ECL,
	0x00008242L,
	0x00008299L,
	0x000082EFL,
	0x00008345L,
	0x0000839CL,
	0x000083F2L,
	0x00008448L,
	0x0000849EL,
	0x000084F4L,
	0x0000854AL,
	0x000085A0L,
	0x000085F5L,
	0x0000864BL,
	0x000086A1L,
	0x000086F6L,
	0x0000874BL,
	0x000087A1L,
	0x000087F6L,
	0x0000884BL,
	0x000088A0L,
	0x000088F5L,
	0x0000894AL,
	0x0000899FL,
	0x000089F4L,
	0x00008A48L,
	0x00008A9DL,
	0x00008AF1L,
	0x00008B46L,
	0x00008B9AL,
	0x00008BEEL,
	0x00008C42L,
	0x00008C96L,
	0x00008CEAL,
	0x00008D3EL,
	0x00008D92L,
	0x00008DE6L,
	0x00008E39L,
	0x00008E8DL,
	0x00008EE0L,
	0x00008F34L,
	0x00008F87L,
	0x00008FDAL,
	0x0000902DL,
	0x00009080L,
	0x000090D3L,
	0x00009126L,
	0x00009179L,
	0x000091CCL,
	0x0000921EL,
	0x00009271L,
	0x000092C3L,
	0x00009315L,
	0x00009368L,
	0x000093BAL,
	0x0000940CL,
	0x0000945EL,
	0x000094B0L,
	0x00009502L,
	0x00009553L,
	0x000095A5L,
	0x000095F6L,
	0x00009648L,
	0x00009699L,
	0x000096EAL,
	0x0000973CL,
	0x0000978DL,
	0x000097DEL,
	0x0000982EL,
	0x0000987FL,
	0x000098D0L,
	0x00009921L,
	0x00009971L,
	0x000099C2L,
	0x00009A12L,
	0x00009A62L,
	0x00009AB2L,
	0x00009B02L,
	0x00009B52L,
	0x00009BA2L,
	0x00009BF2L,
	0x00009C42L,
	0x00009C91L,
	0x00009CE1L,
	0x00009D30L,
	0x00009D7FL,
	0x00009DCFL,
	0x00009E1EL,
	0x00009E6DL,
	0x00009EBCL,
	0x00009F0AL,
	0x00009F59L,
	0x00009FA8L,
	0x00009FF6L,
	0x0000A045L,
	0x0000A093L,
	0x0000A0E1L,
	0x0000A12FL,
	0x0000A17EL,
	0x0000A1CBL,
	0x0000A219L,
	0x0000A267L,
	0x0000A2B5L,
	0x0000A302L,
	0x0000A350L,
	0x0000A39DL,
	0x0000A3EAL,
	0x0000A438L,
	0x0000A485L,
	0x0000A4D2L,
	0x0000A51FL,
	0x0000A56BL,
	0x0000A5B8L,
	0x0000A605L,
	0x0000A651L,
	0x0000A69DL,
	0x0000A6EAL,
	0x0000A736L,
	0x0000A782L,
	0x0000A7CEL,
	0x0000A81AL,
	0x0000A866L,
	0x0000A8B1L,
	0x0000A8FDL,
	0x0000A948L,
	0x0000A994L,
	0x0000A9DFL,
	0x0000AA2AL,
	0x0000AA75L,
	0x0000AAC0L,
	0x0000AB0BL,
	0x0000AB56L,
	0x0000ABA0L,
	0x0000ABEBL,
	0x0000AC35L,
	0x0000AC80L,
	0x0000ACCAL,
	0x0000AD14L,
	0x0000AD5EL,
	0x0000ADA8L,
	0x0000ADF2L,
	0x0000AE3BL,
	0x0000AE85L,
	0x0000AECEL,
	0x0000AF18L,
	0x0000AF61L,
	0x0000AFAAL,
	0x0000AFF3L,
	0x0000B03CL,
	0x0000B085L,
	0x0000B0CEL,
	0x0000B117L,
	0x0000B15FL,
	0x0000B1A8L,
	0x0000B1F0L,
	0x0000B238L,
	0x0000B280L,
	0x0000B2C8L,
	0x0000B310L,
	0x0000B358L,
	0x0000B3A0L,
	0x0000B3E7L,
	0x0000B42FL,
	0x0000B476L,
	0x0000B4BDL,
	0x0000B504L,
	0x0000B54BL,
	0x0000B592L,
	0x0000B5D9L,
	0x0000B620L,
	0x0000B667L,
	0x0000B6ADL,
	0x0000B6F3L,
	0x0000B73AL,
	0x0000B780L,
	0x0000B7C6L,
	0x0000B80CL,
	0x0000B852L,
	0x0000B897L,
	0x0000B8DDL,
	0x0000B922L,
	0x0000B968L,
	0x0000B9ADL,
	0x0000B9F2L,
	0x0000BA37L,
	0x0000BA7CL,
	0x0000BAC1L,
	0x0000BB06L,
	0x0000BB4AL,
	0x0000BB8FL,
	0x0000BBD3L,
	0x0000BC17L,
	0x0000BC5BL,
	0x0000BCA0L,
	0x0000BCE3L,
	0x0000BD27L,
	0x0000BD6BL,
	0x0000BDAEL,
	0x0000BDF2L,
	0x0000BE35L,
	0x0000BE78L,
	0x0000BEBCL,
	0x0000BEFFL,
	0x0000BF41L,
	0x0000BF84L,
	0x0000BFC7L,
	0x0000C009L,
	0x0000C04CL,
	0x0000C08EL,
	0x0000C0D0L,
	0x0000C112L,
	0x0000C154L,
	0x0000C196L,
	0x0000C1D8L,
	0x0000C21AL,
	0x0000C25BL,
	0x0000C29CL,
	0x0000C2DEL,
	0x0000C31FL,
	0x0000C360L,
	0x0000C3A1L,
	0x0000C3E2L,
	0x0000C422L,
	0x0000C463L,
	0x0000C4A3L,
	0x0000C4E3L,
	0x0000C524L,
	0x0000C564L,
	0x0000C5A4L,
	0x0000C5E4L,
	0x0000C623L,
	0x0000C663L,
	0x0000C6A2L,
	0x0000C6E2L,
	0x0000C721L,
	0x0000C760L,
	0x0000C79FL,
	0x0000C7DEL,
	0x0000C81DL,
	0x0000C85BL,
	0x0000C89AL,
	0x0000C8D8L,
	0x0000C916L,
	0x0000C955L,
	0x0000C993L,
	0x0000C9D1L,
	0x0000CA0EL,
	0x0000CA4CL,
	0x0000CA8AL,
	0x0000CAC7L,
	0x0000CB04L,
	0x0000CB41L,
	0x0000CB7FL,
	0x0000CBBBL,
	0x0000CBF8L,
	0x0000CC35L,
	0x0000CC72L,
	0x0000CCAEL,
	0x0000CCEAL,
	0x0000CD26L,
	0x0000CD63L,
	0x0000CD9FL,
	0x0000CDDAL,
	0x0000CE16L,
	0x0000CE52L,
	0x0000CE8DL,
	0x0000CEC8L,
	0x0000CF04L,
	0x0000CF3FL,
	0x0000CF7AL,
	0x0000CFB4L,
	0x0000CFEFL,
	0x0000D02AL,
	0x0000D064L,
	0x0000D09FL,
	0x0000D0D9L,
	0x0000D113L,
	0x0000D14DL,
	0x0000D187L,
	0x0000D1C0L,
	0x0000D1FAL,
	0x0000D233L,
	0x0000D26DL,
	0x0000D2A6L,
	0x0000D2DFL,
	0x0000D318L,
	0x0000D351L,
	0x0000D389L,
	0x0000D3C2L,
	0x0000D3FAL,
	0x0000D433L,
	0x0000D46BL,
	0x0000D4A3L,
	0x0000D4DBL,
	0x0000D512L,
	0x0000D54AL,
	0x0000D582L,
	0x0000D5B9L,
	0x0000D5F0L,
	0x0000D627L,
	0x0000D65FL,
	0x0000D695L,
	0x0000D6CCL,
	0x0000D703L,
	0x0000D739L,
	0x0000D770L,
	0x0000D7A6L,
	0x0000D7DCL,
	0x0000D812L,
	0x0000D848L,
	0x0000D87EL,
	0x0000D8B3L,
	0x0000D8E9L,
	0x0000D91EL,
	0x0000D953L,
	0x0000D988L,
	0x0000D9BDL,
	0x0000D9F2L,
	0x0000DA27L,
	0x0000DA5BL,
	0x0000DA90L,
	0x0000DAC4L,
	0x0000DAF8L,
	0x0000DB2CL,
	0x0000DB60L,
	0x0000DB94L,
	0x0000DBC7L,
	0x0000DBFBL,
	0x0000DC2EL,
	0x0000DC61L,
	0x0000DC94L,
	0x0000DCC7L,
	0x0000DCFAL,
	0x0000DD2DL,
	0x0000DD5FL,
	0x0000DD92L,
	0x0000DDC4L,
	0x0000DDF6L,
	0x0000DE28L,
	0x0000DE5AL,
	0x0000DE8CL,
	0x0000DEBEL,
	0x0000DEEFL,
	0x0000DF20L,
	0x0000DF52L,
	0x0000DF83L,
	0x0000DFB4L,
	0x0000DFE4L,
	0x0000E015L,
	0x0000E046L,
	0x0000E076L,
	0x0000E0A6L,
	0x0000E0D6L,
	0x0000E106L,
	0x0000E136L,
	0x0000E166L,
	0x0000E196L,
	0x0000E1C5L,
	0x0000E1F4L,
	0x0000E224L,
	0x0000E253L,
	0x0000E282L,
	0x0000E2B0L,
	0x0000E2DFL,
	0x0000E30DL,
	0x0000E33CL,
	0x0000E36AL,
	0x0000E398L,
	0x0000E3C6L,
	0x0000E3F4L,
	0x0000E422L,
	0x0000E44FL,
	0x0000E47DL,
	0x0000E4AAL,
	0x0000E4D7L,
	0x0000E504L,
	0x0000E531L,
	0x0000E55EL,
	0x0000E58AL,
	0x0000E5B7L,
	0x0000E5E3L,
	0x0000E60FL,
	0x0000E63BL,
	0x0000E667L,
	0x0000E693L,
	0x0000E6BEL,
	0x0000E6EAL,
	0x0000E715L,
	0x0000E740L,
	0x0000E76BL,
	0x0000E796L,
	0x0000E7C1L,
	0x0000E7ECL,
	0x0000E816L,
	0x0000E841L,
	0x0000E86BL,
	0x0000E895L,
	0x0000E8BFL,
	0x0000E8E9L,
	0x0000E912L,
	0x0000E93CL,
	0x0000E965L,
	0x0000E98EL,
	0x0000E9B7L,
	0x0000E9E0L,
	0x0000EA09L,
	0x0000EA32L,
	0x0000EA5AL,
	0x0000EA83L,
	0x0000EAABL,
	0x0000EAD3L,
	0x0000EAFBL,
	0x0000EB23L,
	0x0000EB4BL,
	0x0000EB72L,
	0x0000EB99L,
	0x0000EBC1L,
	0x0000EBE8L,
	0x0000EC0FL,
	0x0000EC36L,
	0x0000EC5CL,
	0x0000EC83L,
	0x0000ECA9L,
	0x0000ECD0L,
	0x0000ECF6L,
	0x0000ED1CL,
	0x0000ED41L,
	0x0000ED67L,
	0x0000ED8DL,
	0x0000EDB2L,
	0x0000EDD7L,
	0x0000EDFCL,
	0x0000EE21L,
	0x0000EE46L,
	0x0000EE6BL,
	0x0000EE8FL,
	0x0000EEB4L,
	0x0000EED8L,
	0x0000EEFCL,
	0x0000EF20L,
	0x0000EF44L,
	0x0000EF68L,
	0x0000EF8BL,
	0x0000EFAFL,
	0x0000EFD2L,
	0x0000EFF5L,
	0x0000F018L,
	0x0000F03BL,
	0x0000F05DL,
	0x0000F080L,
	0x0000F0A2L,
	0x0000F0C5L,
	0x0000F0E7L,
	0x0000F109L,
	0x0000F12AL,
	0x0000F14CL,
	0x0000F16DL,
	0x0000F18FL,
	0x0000F1B0L,
	0x0000F1D1L,
	0x0000F1F2L,
	0x0000F213L,
	0x0000F233L,
	0x0000F254L,
	0x0000F274L,
	0x0000F294L,
	0x0000F2B5L,
	0x0000F2D4L,
	0x0000F2F4L,
	0x0000F314L,
	0x0000F333L,
	0x0000F353L,
	0x0000F372L,
	0x0000F391L,
	0x0000F3B0L,
	0x0000F3CEL,
	0x0000F3EDL,
	0x0000F40BL,
	0x0000F42AL,
	0x0000F448L,
	0x0000F466L,
	0x0000F484L,
	0x0000F4A1L,
	0x0000F4BFL,
	0x0000F4DCL,
	0x0000F4FAL,
	0x0000F517L,
	0x0000F534L,
	0x0000F550L,
	0x0000F56DL,
	0x0000F58AL,
	0x0000F5A6L,
	0x0000F5C2L,
	0x0000F5DEL,
	0x0000F5FAL,
	0x0000F616L,
	0x0000F632L,
	0x0000F64DL,
	0x0000F668L,
	0x0000F684L,
	0x0000F69FL,
	0x0000F6BAL,
	0x0000F6D4L,
	0x0000F6EFL,
	0x0000F709L,
	0x0000F724L,
	0x0000F73EL,
	0x0000F758L,
	0x0000F772L,
	0x0000F78BL,
	0x0000F7A5L,
	0x0000F7BEL,
	0x0000F7D7L,
	0x0000F7F1L,
	0x0000F80AL,
	0x0000F822L,
	0x0000F83BL,
	0x0000F853L,
	0x0000F86CL,
	0x0000F884L,
	0x0000F89CL,
	0x0000F8B4L,
	0x0000F8CCL,
	0x0000F8E3L,
	0x0000F8FBL,
	0x0000F912L,
	0x0000F929L,
	0x0000F940L,
	0x0000F957L,
	0x0000F96EL,
	0x0000F984L,
	0x0000F99BL,
	0x0000F9B1L,
	0x0000F9C7L,
	0x0000F9DDL,
	0x0000F9F3L,
	0x0000FA09L,
	0x0000FA1EL,
	0x0000FA33L,
	0x0000FA49L,
	0x0000FA5EL,
	0x0000FA73L,
	0x0000FA87L,
	0x0000FA9CL,
	0x0000FAB0L,
	0x0000FAC5L,
	0x0000FAD9L,
	0x0000FAEDL,
	0x0000FB01L,
	0x0000FB14L,
	0x0000FB28L,
	0x0000FB3BL,
	0x0000FB4EL,
	0x0000FB61L,
	0x0000FB74L,
	0x0000FB87L,
	0x0000FB9AL,
	0x0000FBACL,
	0x0000FBBFL,
	0x0000FBD1L,
	0x0000FBE3L,
	0x0000FBF5L,
	0x0000FC06L,
	0x0000FC18L,
	0x0000FC29L,
	0x0000FC3BL,
	0x0000FC4CL,
	0x0000FC5DL,
	0x0000FC6EL,
	0x0000FC7EL,
	0x0000FC8FL,
	0x0000FC9FL,
	0x0000FCAFL,
	0x0000FCBFL,
	0x0000FCCFL,
	0x0000FCDFL,
	0x0000FCEFL,
	0x0000FCFEL,
	0x0000FD0DL,
	0x0000FD1CL,
	0x0000FD2BL,
	0x0000FD3AL,
	0x0000FD49L,
	0x0000FD57L,
	0x0000FD66L,
	0x0000FD74L,
	0x0000FD82L,
	0x0000FD90L,
	0x0000FD9EL,
	0x0000FDABL,
	0x0000FDB9L,
	0x0000FDC6L,
	0x0000FDD3L,
	0x0000FDE0L,
	0x0000FDEDL,
	0x0000FDFAL,
	0x0000FE06L,
	0x0000FE13L,
	0x0000FE1FL,
	0x0000FE2BL,
	0x0000FE37L,
	0x0000FE43L,
	0x0000FE4EL,
	0x0000FE5AL,
	0x0000FE65L,
	0x0000FE70L,
	0x0000FE7BL,
	0x0000FE86L,
	0x0000FE91L,
	0x0000FE9BL,
	0x0000FEA6L,
	0x0000FEB0L,
	0x0000FEBAL,
	0x0000FEC4L,
	0x0000FECEL,
	0x0000FED7L,
	0x0000FEE1L,
	0x0000FEEAL,
	0x0000FEF3L,
	0x0000FEFCL,
	0x0000FF05L,
	0x0000FF0EL,
	0x0000FF16L,
	0x0000FF1FL,
	0x0000FF27L,
	0x0000FF2FL,
	0x0000FF37L,
	0x0000FF3FL,
	0x0000FF46L,
	0x0000FF4EL,
	0x0000FF55L,
	0x0000FF5CL,
	0x0000FF63L,
	0x0000FF6AL,
	0x0000FF71L,
	0x0000FF78L,
	0x0000FF7EL,
	0x0000FF84L,
	0x0000FF8AL,
	0x0000FF90L,
	0x0000FF96L,
	0x0000FF9CL,
	0x0000FFA1L,
	0x0000FFA6L,
	0x0000FFACL,
	0x0000FFB1L,
	0x0000FFB5L,
	0x0000FFBAL,
	0x0000FFBFL,
	0x0000FFC3L,
	0x0000FFC7L,
	0x0000FFCBL,
	0x0000FFCFL,
	0x0000FFD3L,
	0x0000FFD7L,
	0x0000FFDAL,
	0x0000FFDDL,
	0x0000FFE1L,
	0x0000FFE4L,
	0x0000FFE7L,
	0x0000FFE9L,
	0x0000FFECL,
	0x0000FFEEL,
	0x0000FFF0L,
	0x0000FFF2L,
	0x0000FFF4L,
	0x0000FFF6L,
	0x0000FFF8L,
	0x0000FFF9L,
	0x0000FFFBL,
	0x0000FFFCL,
	0x0000FFFDL,
	0x0000FFFEL,
	0x0000FFFEL,
	0x0000FFFFL,
	0x0000FFFFL,
	0x00010000L,
	0x00010000L,
};
/****************************** M A C R O S ******************************/


/**************************** R O U T I N E S ****************************/

static long realRandomNumber	= INIT_STATIC_SEED;
static long pseudoRandomNumber  = INIT_STATIC_SEED;

/******************************* M A C R O S ******************************/


/***************************** R O U T I N E S ****************************/

/*************************************************************************
                                UTL_fmod
 *************************************************************************

   SYNOPSIS
		float UTL_fmod (float v, float d)

   PURPOSE


   INPUT
		v :
		d :

   OUTPUT
		None

   EFFECTS
		None

   RETURNS


   SEE ALSO


   HISTORY
		11/27/95 GAT: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

float UTL_fmod (float v, float d)
BEGINFUNC (UTL_fmod)
{
	float	q;
	long	newQ;

	q    = v / d;
	newQ = (long)q;
	q    = newQ * d;

	RETURN v - q;

} ENDFUNC (UTL_fmod)

/*************************************************************************
                                UTL_ceil
 *************************************************************************

   SYNOPSIS
		float UTL_ceil (float v)

   PURPOSE


   INPUT
		v :

   OUTPUT
		None

   EFFECTS
		None

   RETURNS


   SEE ALSO


   HISTORY
		11/27/95 GAT: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

float UTL_ceil (float v)
BEGINFUNC (UTL_ceil)
{
	if (v < 0)
	{
		long	newV = (long)v;

		RETURN (float)newV;
	}
	else
	{
		long	newV = (long)v;

		if (newV != v)
		{
			RETURN (float)(newV + 1);
		}
	}

	RETURN v;


} ENDFUNC (UTL_ceil)

/*************************************************************************
                                UTL_floor
 *************************************************************************

   SYNOPSIS
		float UTL_floor (float v)

   PURPOSE


   INPUT
		v :

   OUTPUT
		None

   EFFECTS
		None

   RETURNS


   SEE ALSO


   HISTORY
		11/27/95 GAT: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

float UTL_floor (float v)
BEGINFUNC (UTL_floor)
{
	if (v >= 0)
	{
		long	newV = (long)v;

		RETURN (float)newV;
	}
	else
	{
		long	newV = (long)v;

		if (newV != v)
		{
			RETURN (float)(newV - 1);
		}
	}

	RETURN v;

} ENDFUNC (UTL_floor)

/*************************************************************************
                                UTL_SINF
 *************************************************************************

   SYNOPSIS
		float UTL_SINF (float degrees)

   PURPOSE
		return the SIN of the given angle by using UTL_SIN1024 and
		interpolating.

   INPUT
		degrees : 0-360 where 361 = 1 and 721 = 1 and -359 = 1

   OUTPUT
		None

   EFFECTS
		None

   RETURNS


   SEE ALSO


   HISTORY
		11/27/95 GAT: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

float UTL_SINF (float degrees)
BEGINFUNC (UTL_SINF)
{
	float	sinUnits;
	float	sinFloor;
	float	sinCeil;

	sinUnits = degrees * 1024 / 360;
	sinFloor = UTL_SIN1024 (UTL_floor (sinUnits));
	sinCeil  = UTL_SIN1024 (UTL_ceil (sinUnits));

	if (sinFloor > sinCeil)
	{
		RETURN UTL_TERPF (UTL_fmod (sinUnits, (float)1.0), sinCeil, sinFloor);
	}

	RETURN UTL_TERPF (UTL_fmod (sinUnits, (float)1.0), sinFloor, sinCeil);

} ENDFUNC (UTL_SINF)

/*************************************************************************
                           UTL_PseudoRandom16
 *************************************************************************

   SYNOPSIS
		int16	UTL_PseudoRandom16 (void)

   PURPOSE
		Return a random number that can be reproduced

		Call UTL_ResetPseudoRandom16() to start over

   INPUT
		None

   OUTPUT
		None

   EFFECTS
		None

   RETURNS


   SEE ALSO


   HISTORY
		10/12/95 GAT: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int16 UTL_PseudoRandom (void)
BEGINFUNC (UTL_PseudoRandom)
{
	pseudoRandomNumber	= (pseudoRandomNumber * 125) % STATIC_DIV;

	RETURN (int16)(pseudoRandomNumber & 0x00007FFFUL);

} ENDFUNC (UTL_PseudoRandom)

/*************************************************************************
                           UTL_ReallyRandom16
 *************************************************************************

   SYNOPSIS
		int16 UTL_ReallyRandom16 (void)

   PURPOSE
		Return a truly random number (can not be re-produced)

   INPUT
		None

   OUTPUT
		None

   EFFECTS
		None

   RETURNS


   SEE ALSO


   HISTORY
		10/12/95 GAT: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int16 UTL_ReallyRandom (void)
BEGINFUNC (UTL_ReallyRandom)
{
	realRandomNumber	= (realRandomNumber * 125) % STATIC_DIV;

	RETURN (int16)(realRandomNumber & 0x00007FFFUL);

} ENDFUNC (UTL_ReallyRandom)

/*************************************************************************
                       UTL_ResetPseudoRandomNumber
 *************************************************************************

   SYNOPSIS
		void UTL_ResetPseudoRandomNumber (void)

   PURPOSE


   INPUT
		None

   OUTPUT
		None

   EFFECTS
		None

   SEE ALSO


   HISTORY
		10/12/95 GAT: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

void UTL_ResetPseudoRandomNumber (void)
BEGINPROC (UTL_ResetPseudoRandomNumber)
{
	pseudoRandomNumber = INIT_STATIC_SEED;

} ENDPROC (UTL_ResetPseudoRandomNumber)

/*************************************************************************
								   UTL_DIVIDE1616
 *************************************************************************

	SYNOPSIS
		int32 UTL_DIVIDE1616(int32 a, int32 b)

	PURPOSE
		Divide 16.16 fixed point format dividend by 16.16 divisor.

	INPUT
		a	: dividend
		b	: divisor

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		quotient

	SEE ALSO


	HISTORY
		05/05/97 MDW: Created. There has GOT to be a better way to do this.

#define UTL_DIVIDE1616(a,b) ((UTL_ABS(b) >> 8 > 0) ? (((a) / ((b) >> 8)) << 8) : ((UTL_ABS(b)) ? (((a) / (b)) << 16) : 0))

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int32 UTL_DIVIDE1616 (int32 a, int32 b)

BEGINFUNC (UTL_DIVIDE1616)
{
	if (UTL_ABS(b) > 256)
	{
		RETURN ((a / (b >> 8)) << 8);
	}
	else
	{
		RETURN((a / b) << 16);
	}
}
ENDFUNC (UTL_DIVIDE1616)

/*************************************************************************
								  UTL_MULT1616
 *************************************************************************

	SYNOPSIS
		int32 UTL_MULT1616(int32 a, int32 b)

	PURPOSE
		Multiply multiplicand by multiplier.

	INPUT
		a	: mulitplicand
		b	: multiplier

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		result

	SEE ALSO


	HISTORY
		05/05/97 MDW: Created.

#define UTL_MULT1616(a,b)	(((a) >> 8) * ((b) >> 8))

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int32 UTL_MULT1616 (int32 a, int32 b)

BEGINFUNC (UTL_MULT1616)
{
#if SIMPLE_N_EASY

	RETURN((a >> 8) * (b >> 8));

#else

	ENSURE_ ((a > 0xFFFF && b > 0xFFFF), "Fixed Point multiply overflow.");

	if (UTL_ABS(a) < 0x100L)
	{
		if (UTL_ABS(b) < 0x100L)
		{
			RETURN((a * b) >> 16);
		}
		else
		{
			RETURN((a * (b >> 8)) >> 8);
		}
	}
	else
	{
		if (UTL_ABS(b) < 0x100L)
		{
			RETURN(((a >> 8) * b) << 8);
		}
		else
		{
			RETURN((a >> 8) * (b >> 8));
		}
	}

#endif
}
ENDFUNC (UTL_MULT1616)

/*************************************************************************
									 UTL_SIN
 *************************************************************************

	SYNOPSIS
		int32 UTL_SIN(uint8 angle)

	PURPOSE
		Return the sin value for an angle (0-255)

	INPUT
		angle	: index into sin table

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		sin(angle)

	SEE ALSO


	HISTORY
		04/17/97 MDW: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int32 UTL_SIN (uint8 angle)

BEGINFUNC (UTL_SIN)
{
	if (angle >= 192)
	{
		// quadrant IV
		RETURN -gSinTable64[256 - angle];
	}
	else if (angle >= 128)
	{
		// quadrant III
		RETURN -gSinTable64[angle - 128];
	}
	else if (angle >= 64)
	{
		// quadrant II
		RETURN gSinTable64[128 - angle];
	}
	else
	{
		// quadrant I
		RETURN gSinTable64[angle];
	}
}
ENDFUNC (UTL_SIN)

/*************************************************************************
									 UTL_COS
 *************************************************************************

	SYNOPSIS
		int32 UTL_COS(uint8 angle)

	PURPOSE
		Return the cosine value for an angle (0-255)

	INPUT
		angle	: index into sin table

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		cos(angle)

	SEE ALSO


	HISTORY
		04/17/97 MDW: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int32 UTL_COS (uint8 angle)

BEGINFUNC (UTL_COS)
{
	if (angle >= 192)
	{
		// quadrant IV
		RETURN  gSinTable64[angle - 192];
	}
	else if (angle >= 128)
	{
		// quadrant III
		RETURN -gSinTable64[192 - angle];
	}
	else if (angle >= 64)
	{
		// quadrant II
		RETURN -gSinTable64[angle - 64];
	}
	else
	{
		// quadrant I
		RETURN  gSinTable64[64 - angle];
	}
}
ENDFUNC (UTL_COS)

/*************************************************************************
								   UTL_FindAngle
 *************************************************************************

	SYNOPSIS
		uint8 UTL_FindAngle(int32 x, int32 y)

	PURPOSE
		Translates tan, y / x, into the respective angle.
		c.f. Graphics Gems II pp. 389-391

		The basic algorithm is to find the octant which contains the angle
		then use the (a < 1) property of the tangent in that octant to
		interpolate the angle from the resulting fractional part of an octant.

		This function has been specialized for the current use of 256 angles
		per circle, so each octant can have	32 possible angles inside.

	INPUT
		x	: x-coord of line in 16.16 fixed point format
		y	: y-coord of line in 16.16 fixed point format

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		angle	: 0-255

	SEE ALSO


	HISTORY
		04/19/97 MDW: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
uint8 UTL_FindAngle (int32 x, int32 y)

BEGINFUNC (UTL_FindAngle)
{
	int32 dx = UTL_ABS(x), dy = UTL_ABS(y);

	// handle special cases
	if (x == 0 || y == 0)
	{
		if (x == 0)
		{
			if (y > 0)
			{
				RETURN(64);
			}
			else
			{
				RETURN(192);
			}
		}
		else
		{
			if (x > 0)
			{
				RETURN(0);
			}
			else
			{
				RETURN(128);
			}

		}
	}
	if (dx == dy)
	{
		if (dx > 0)
		{
			if (dy > 0)
			{
				RETURN(32);
			}
			else
			{
				RETURN(240);
			}
		}
		else
		{
			if (dy > 0)
			{
				RETURN(96);
			}
			else
			{
				RETURN(160);
			}
		}
	}
	// locate proper octant for angle
	if (x > 0)
	{
		if (y > 0)
		{
			if (dx > dy)
			{
				// angle = ((octant * 32) + (tan * 32)  -- (32 angle values per octant)
				// octant numbers run counterclockwise (like an angle through a circle
				// tan part is always < 1

				// octant0
				RETURN (uint8) (0x00000000L + (UTL_DIVIDE1616 (dy, dx) << 5) >> 16);
			}
			else
			{
				// octant1
				RETURN (uint8) (0x00200000L + ((0x00010000L - UTL_DIVIDE1616 (dx, dy)) << 5) >> 16);
			}
		}
		else
		{
			if (dx > dy)
			{
				// octant7
				RETURN (uint8) (0x00e00000L + ((0x00010000L - UTL_DIVIDE1616 (dy, dx)) << 5) >> 16);
			}
			else
			{
				// octant6
				RETURN (uint8) (0x00c00000L + (UTL_DIVIDE1616 (dx, dy) << 5) >> 16);
			}
		}
	}
	else
	{
		if (y > 0)
		{
			if (dx > dy)
			{
				// octant3
				RETURN (uint8) (0x00600000L + ((0x00010000L - UTL_DIVIDE1616 (dy, dx)) << 5) >> 16);
			}
			else
			{
				// octant2
				RETURN (uint8) (0x00400000L + (UTL_DIVIDE1616 (dx, dy) << 5) >> 16);
			}
		}
		else
		{
			if (dx > dy)
			{
				// octant4
				RETURN (uint8) (0x00800000L + (UTL_DIVIDE1616 (dy, dx) << 5) >> 16);
			}
			else
			{
				// octant5
				RETURN (uint8) (0x00a00000L + ((0x00010000L - UTL_DIVIDE1616 (dx, dy)) << 5) >> 16);
			}
		}
	}
}
ENDFUNC (UTL_FindAngle)


/*************************************************************************
								UTL_Sin4096
 *************************************************************************

	SYNOPSIS
		int32 UTL_Sin4096(int32 angle)

	PURPOSE
		Return the sin value for an angle (0-4096)

	INPUT
		angle	: index into sin table

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		sin(angle)

	SEE ALSO


	HISTORY
		04/17/97 MDW: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int32 UTL_Sin4096 (int32 angle)

BEGINFUNC (UTL_Sin4096)
{
	angle &= 4096 - 1;

	if (angle >= 3072)
	{
		// quadrant IV
		RETURN -SinTable1024[4096 - angle];
	}
	else if (angle >= 2048)
	{
		// quadrant III
		RETURN -SinTable1024[angle - 2048];
	}
	else if (angle >= 1024)
	{
		// quadrant II
		RETURN SinTable1024[2048 - angle];
	}
	else
	{
		// quadrant I
		RETURN SinTable1024[angle];
	}
}
ENDFUNC (UTL_Sin4096)

/*************************************************************************
								UTL_Cos4096
 *************************************************************************

	SYNOPSIS
		int32 UTL_Cos4096(int32 angle)

	PURPOSE
		Return the cosine value for an angle (0-4096)

	INPUT
		angle	: index into sin table

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		cos(angle)

	SEE ALSO


	HISTORY
		04/17/97 MDW: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int32 UTL_Cos4096 (int32 angle)

BEGINFUNC (UTL_Cos4096)
{
	angle &= 4096 - 1;

	if (angle >= 3072)
	{
		// quadrant IV
		RETURN  SinTable1024[angle - 3072];
	}
	else if (angle >= 2048)
	{
		// quadrant III
		RETURN -SinTable1024[3072 - angle];
	}
	else if (angle >= 1024)
	{
		// quadrant II
		RETURN -SinTable1024[angle - 1024];
	}
	else
	{
		// quadrant I
		RETURN  SinTable1024[1024 - angle];
	}
}
ENDFUNC (UTL_Cos4096)

/*************************************************************************
								   UTL_FindAngle4096
 *************************************************************************

	SYNOPSIS
		int32 UTL_FindAngle4096(int32 x, int32 y)

	PURPOSE
		Translates tan, y / x, into the respective angle.
		c.f. Graphics Gems II pp. 389-391

		The basic algorithm is to find the octant which contains the angle
		then use the (a < 1) property of the tangent in that octant to
		interpolate the angle from the resulting fractional part of an octant.

		This function has been specialized for the current use of 4096 angles
		per circle, so each octant can have	512 possible angles inside.

	INPUT
		x	: x-coord of line in 16.16 fixed point format
		y	: y-coord of line in 16.16 fixed point format

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		angle	: 0-4096

	SEE ALSO


	HISTORY
		06/03/97 MDW: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int32 UTL_FindAngle4096 (int32 x, int32 y)

BEGINFUNC (UTL_FindAngle4096)
{
	int32 dx = UTL_ABS(x), dy = UTL_ABS(y);

	// handle special case axis alignment
	if (x == 0 || y == 0)
	{
		if (x == 0)
		{
			if (y > 0)
			{
				RETURN(1023);
			}
			else
			{
				RETURN(3071);
			}
		}
		else
		{
			if (x > 0)
			{
				RETURN(0);
			}
			else
			{
				RETURN(2047);
			}

		}
	}
	// handle special case 45 degree angles
	if (dx == dy)
	{
		if (x > 0)
		{
			if (y > 0)
			{
				RETURN(511);
			}
			else
			{
				RETURN(3583);
			}
		}
		else
		{
			if (y > 0)
			{
				RETURN(1535);
			}
			else
			{
				RETURN(2559);
			}
		}
	}
	// otherwise locate proper octant for angle
	if (x > 0)
	{
		if (y > 0)
		{
			if (dx > dy)
			{
				// angle = ((octant * 512) + (tan * 512)  -- (512 angle values per octant)
				// octant numbers run counterclockwise (like an angle through a circle
				// tan part is always < 1

				// octant0
				RETURN (0x00000000L + (UTL_DIVIDE1616 (dy, dx) << 9) >> 16);
			}
			else
			{
				// octant1
				RETURN (0x02000000L + ((0x00010000L - UTL_DIVIDE1616 (dx, dy)) << 9) >> 16);
			}
		}
		else
		{
			if (dx > dy)
			{
				// octant7
				RETURN (0x0e000000L + ((0x00010000L - UTL_DIVIDE1616 (dy, dx)) << 9) >> 16);
			}
			else
			{
				// octant6
				RETURN (0x0c000000L + (UTL_DIVIDE1616 (dx, dy) << 9) >> 16);
			}
		}
	}
	else
	{
		if (y > 0)
		{
			if (dx > dy)
			{
				// octant3
				RETURN (0x06000000L + ((0x00010000L - UTL_DIVIDE1616 (dy, dx)) << 9) >> 16);
			}
			else
			{
				// octant2
				RETURN (0x04000000L + (UTL_DIVIDE1616 (dx, dy) << 9) >> 16);
			}
		}
		else
		{
			if (dx > dy)
			{
				// octant4
				RETURN (0x08000000L + (UTL_DIVIDE1616 (dy, dx) << 9) >> 16);
			}
			else
			{
				// octant5
				RETURN (0x0a000000L + ((0x00010000L - UTL_DIVIDE1616 (dx, dy)) << 9) >> 16);
			}
		}
	}
}
ENDFUNC (UTL_FindAngle4096)

/*************************************************************************
							UTL_AngleToDirection4096
 *************************************************************************

	SYNOPSIS
		int8 UTL_AngleToDirection4096(int32 angle)

	PURPOSE
		Convert specified angle to one of eight directions. Assumes angle
		was generated by one of the 4096 series functions.

	INPUT
		None

	OUTPUT
		None

	EFFECTS
		None

	RETURNS
		None

	SEE ALSO


	HISTORY
		06/24/97 MDW: Created.

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#define DIR_NORTH		0
#define DIR_NORTHEAST	1
#define DIR_EAST		2
#define DIR_SOUTHEAST	3
#define DIR_SOUTH		4
#define DIR_SOUTHWEST	5
#define DIR_WEST		6
#define DIR_NORTHWEST	7

int8 UTL_AngleToDirection4096 (int32 angle)
BEGINFUNC (UTL_AngleToDirection)
{
	// use decision tree to convert angle to facing direction
	if (1280 >= angle && angle > 768)
	{
		RETURN DIR_NORTH;
	}
	if (768 >= angle && angle > 256)
	{
		RETURN DIR_NORTHEAST;
	}
	if (256 >= angle || angle > 3840)
	{
		RETURN DIR_EAST;
	}
	if (3840 >= angle && angle > 3328)
	{
		RETURN DIR_SOUTHEAST;
	}
	if (3328 >= angle && angle > 2816)
	{
		RETURN DIR_SOUTH;
	}
	if (2816 >= angle && angle > 2304)
	{
		RETURN DIR_SOUTHWEST;
	}
	if (2304 >= angle && angle > 1792)
	{
		RETURN DIR_WEST;
	}
	if (1792 >= angle && angle > 1280)
	{
		RETURN DIR_NORTHWEST;
	}
}
ENDFUNC (UTL_AngleToDirection4096)
